#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from marshmallow.exceptions import ValidationError

from polyaxon.polyflow.matrix.bayes import (
    AcquisitionFunctions,
    GaussianProcessesKernels,
)
from polyaxon.polyflow.operations import V1CompiledOperation
from polyaxon.polyflow.optimization import V1Optimization, V1OptimizationMetric
from polyaxon.polyflow.run.kinds import V1RunKind
from tests.utils import BaseTestCase, assert_equal_dict


@pytest.mark.workflow_mark
class TestWorkflowConfigs(BaseTestCase):
    def test_workflow_config_raise_conditions(self):
        config_dict = {
            "kind": "compiled_operation",
            "matrix": {
                "kind": "mapping",
                "concurrency": 2,
                "values": [{"foo": 1}, {"foo": 2}, {"foo": 3}],
            },
            "run": {"kind": V1RunKind.JOB, "container": {"image": "test"}},
        }
        config = V1CompiledOperation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        # Add random_search without matrix should raise
        config_dict["matrix"] = {"kind": "random", "numRuns": 10}
        with self.assertRaises(ValidationError):
            V1CompiledOperation.from_dict(config_dict)

        # Add a matrix definition with 2 methods
        config_dict["matrix"]["params"] = {
            "lr": {
                "kind": "choice",
                "value": [1, 2, 3],
                "pvalues": [(1, 0.3), (2, 0.3), (3, 0.3)],
            }
        }
        with self.assertRaises(ValidationError):
            V1CompiledOperation.from_dict(config_dict)

        # Using a distribution with random search should pass
        config_dict["matrix"]["params"] = {
            "lr": {"kind": "pchoice", "value": [(1, 0.3), (2, 0.3), (3, 0.3)]}
        }
        config = V1CompiledOperation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        # Add matrix definition should pass
        config_dict["matrix"]["params"] = {"lr": {"kind": "choice", "value": [1, 2, 3]}}
        config = V1CompiledOperation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        # Add grid_search should raise
        config_dict["matrix"] = {"kind": "grid", "numRuns": 10}
        config_dict["matrix"]["params"] = {"lr": {"kind": "choice", "value": [1, 2, 3]}}
        config = V1CompiledOperation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        # Adding a distribution should raise
        config_dict["matrix"]["params"] = {
            "lr": {"kind": "pchoice", "value": [(1, 0.3), (2, 0.3), (3, 0.3)]}
        }
        with self.assertRaises(ValidationError):
            V1CompiledOperation.from_dict(config_dict)

        # Updating the matrix should pass
        config_dict["matrix"]["params"] = {"lr": {"kind": "choice", "value": [1, 2, 3]}}
        config = V1CompiledOperation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        # Add hyperband should raise
        config_dict["matrix"] = {
            "kind": "hyperband",
            "maxIterations": 10,
            "eta": 3,
            "resource": {"name": "steps", "type": "int"},
            "resume": False,
            "metric": V1OptimizationMetric(
                name="loss", optimization=V1Optimization.MINIMIZE
            ).to_dict(),
            "params": {
                "lr": {"kind": "pchoice", "value": [(1, 0.3), (2, 0.3), (3, 0.3)]}
            },
            "seed": 1,
        }
        config = V1CompiledOperation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        # Add early stopping
        config_dict["matrix"]["earlyStopping"] = [
            {
                "kind": "metric_early_stopping",
                "metric": "loss",
                "value": 0.1,
                "optimization": V1Optimization.MINIMIZE,
                "policy": {"kind": "median", "evaluationInterval": 1},
            },
            {
                "kind": "metric_early_stopping",
                "metric": "accuracy",
                "value": 0.9,
                "optimization": V1Optimization.MAXIMIZE,
                "policy": {
                    "kind": "truncation",
                    "percent": 50,
                    "evaluationInterval": 1,
                },
            },
        ]
        config = V1CompiledOperation.from_dict(config_dict)
        assert config.to_dict() == config_dict

        # Add bayes should raise
        config_dict["matrix"] = {
            "kind": "bayes",
            "metric": V1OptimizationMetric(
                name="loss", optimization=V1Optimization.MINIMIZE
            ).to_dict(),
            "numInitialRuns": 2,
            "maxIterations": 10,
            "utilityFunction": {
                "acquisitionFunction": AcquisitionFunctions.UCB,
                "kappa": 1.2,
                "gaussianProcess": {
                    "kernel": GaussianProcessesKernels.MATERN,
                    "lengthScale": 1.0,
                    "nu": 1.9,
                    "numRestartsOptimizer": 2,
                },
            },
            "params": {
                "lr": {"kind": "pchoice", "value": [(1, 0.3), (2, 0.3), (3, 0.3)]}
            },
            "seed": 1,
        }
        with self.assertRaises(ValidationError):
            V1CompiledOperation.from_dict(config_dict)

        # Using non uniform distribution should raise
        # Updating the matrix should pass
        config_dict["matrix"]["params"] = {
            "lr": {"kind": "pchoice", "value": [[0.1, 0.1], [0.2, 0.9]]}
        }
        with self.assertRaises(ValidationError):
            V1CompiledOperation.from_dict(config_dict)

        config_dict["matrix"]["params"] = {
            "lr": {"kind": "normal", "value": [0.1, 0.2]}
        }
        with self.assertRaises(ValidationError):
            V1CompiledOperation.from_dict(config_dict)

        # Using uniform distribution should not raise
        config_dict["matrix"]["params"] = {
            "lr": {"kind": "uniform", "value": {"low": 0.1, "high": 0.2}}
        }
        config = V1CompiledOperation.from_dict(config_dict)
        assert_equal_dict(config.to_dict(), config_dict)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from polyaxon.agents.spawners.async_spawner import AsyncSpawner
from polyaxon.exceptions import PolyaxonAgentError
from polyaxon.polyflow import V1RunKind
from tests.utils import AsyncMock


@pytest.mark.asyncio
@pytest.mark.filterwarnings("ignore::RuntimeWarning")
async def test_start_apply_stop_get_raises_for_non_recognized_kinds():
    spawner = AsyncSpawner()

    try:
        await spawner.create(run_uuid="", run_kind="foo", resource={})
    except PolyaxonAgentError:
        assert True

    try:
        await spawner.apply(run_uuid="", run_kind="foo", resource={})
    except PolyaxonAgentError:
        assert True

    try:
        await spawner.stop(run_uuid="", run_kind="foo")
    except PolyaxonAgentError:
        assert True

    try:
        await spawner.get(run_uuid="", run_kind="foo")
    except PolyaxonAgentError:
        assert True


@pytest.mark.asyncio
@pytest.mark.filterwarnings("ignore::RuntimeWarning")
async def test_start_apply_stop_get():
    class k8s_manager:
        create_custom_object = AsyncMock()
        update_custom_object = AsyncMock()
        get_custom_object = AsyncMock()
        delete_custom_object = AsyncMock()

    spawner = AsyncSpawner()
    k8s_manager.create_custom_object.return_value = ("", "")
    spawner._k8s_manager = k8s_manager

    await spawner.create(run_uuid="", run_kind=V1RunKind.JOB, resource={})
    assert k8s_manager.create_custom_object.call_count == 1

    await spawner.apply(run_uuid="", run_kind=V1RunKind.JOB, resource={})
    assert k8s_manager.update_custom_object.call_count == 1

    await spawner.stop(run_uuid="", run_kind=V1RunKind.JOB)
    assert k8s_manager.delete_custom_object.call_count == 1

    await spawner.get(run_uuid="", run_kind=V1RunKind.JOB)
    assert k8s_manager.get_custom_object.call_count == 1



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.polyflow import V1RunKind
from polyaxon.polypod.compiler.converters.base import BaseConverter, ConverterAbstract
from polyaxon.polypod.compiler.converters.helpers import (
    CleanerConverter,
    NotifierConverter,
    PlatformCleanerConverter,
    PlatformNotifierConverter,
    PlatformTunerConverter,
    PlatformWatchDogConverter,
    TunerConverter,
    WatchDogConverter,
)
from polyaxon.polypod.compiler.converters.job import JobConverter, PlatformJobConverter
from polyaxon.polypod.compiler.converters.kubeflow import (
    MPIJobConverter,
    PytorchJobConverter,
    TfJobConverter,
)
from polyaxon.polypod.compiler.converters.kubeflow.mpi_job import (
    PlatformMPIJobConverter,
)
from polyaxon.polypod.compiler.converters.kubeflow.pytroch_job import (
    PlatformPytorchJobConverter,
)
from polyaxon.polypod.compiler.converters.kubeflow.tf_job import PlatformTfJobConverter
from polyaxon.polypod.compiler.converters.service import (
    PlatformServiceConverter,
    ServiceConverter,
)

CORE_CONVERTERS = {
    V1RunKind.NOTIFIER: NotifierConverter,
    V1RunKind.CLEANER: CleanerConverter,
    V1RunKind.TUNER: TunerConverter,
    V1RunKind.WATCHDOG: WatchDogConverter,
    V1RunKind.JOB: JobConverter,
    V1RunKind.SERVICE: ServiceConverter,
    V1RunKind.MPIJOB: MPIJobConverter,
    V1RunKind.TFJOB: TfJobConverter,
    V1RunKind.PYTORCHJOB: PytorchJobConverter,
}

PLATFORM_CONVERTERS = {
    V1RunKind.NOTIFIER: PlatformNotifierConverter,
    V1RunKind.CLEANER: PlatformCleanerConverter,
    V1RunKind.TUNER: PlatformTunerConverter,
    V1RunKind.WATCHDOG: PlatformWatchDogConverter,
    V1RunKind.JOB: PlatformJobConverter,
    V1RunKind.SERVICE: PlatformServiceConverter,
    V1RunKind.MPIJOB: PlatformMPIJobConverter,
    V1RunKind.TFJOB: PlatformTfJobConverter,
    V1RunKind.PYTORCHJOB: PlatformPytorchJobConverter,
}



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import pytest
import tempfile

from polyaxon.proxies.generators import (
    generate_api_conf,
    generate_gateway_conf,
    generate_streams_conf,
)
from tests.utils import BaseTestCase


@pytest.mark.proxies_mark
class TestGenerate(BaseTestCase):
    SET_PROXIES_SETTINGS = True

    def test_generate_api_conf(self):
        tmp_dir = tempfile.mkdtemp()
        assert os.listdir(tmp_dir) == []
        generate_api_conf(path=tmp_dir)
        assert set(os.listdir(tmp_dir)) == {"polyaxon.main.conf", "polyaxon.base.conf"}

    def test_generate_gateway_conf(self):
        tmp_dir = tempfile.mkdtemp()
        assert os.listdir(tmp_dir) == []
        generate_gateway_conf(path=tmp_dir)
        assert set(os.listdir(tmp_dir)) == {
            "polyaxon.main.conf",
            "polyaxon.base.conf",
            "polyaxon.redirect.conf",
        }

    def test_generate_streams_conf(self):
        tmp_dir = tempfile.mkdtemp()
        assert os.listdir(tmp_dir) == []
        generate_streams_conf(path=tmp_dir)
        assert set(os.listdir(tmp_dir)) == {"polyaxon.main.conf", "polyaxon.base.conf"}



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.schemas.types.artifacts import ArtifactsTypeSchema, V1ArtifactsType
from polyaxon.schemas.types.auth import AuthTypeSchema, V1AuthType
from polyaxon.schemas.types.connections import ConnectionTypeSchema, V1ConnectionType
from polyaxon.schemas.types.dockerfile import DockerfileTypeSchema, V1DockerfileType
from polyaxon.schemas.types.gcs import GcsTypeSchema, V1GcsType
from polyaxon.schemas.types.git import GitTypeSchema, V1GitType
from polyaxon.schemas.types.k8s_resources import (
    K8sResourceTypeSchema,
    V1K8sResourceType,
)
from polyaxon.schemas.types.s3 import S3TypeSchema, V1S3Type
from polyaxon.schemas.types.uri import UriTypeSchema, V1UriType
from polyaxon.schemas.types.wasb import V1WasbType, WasbTypeSchema



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import pytest

from polyaxon.k8s import k8s_schemas
from polyaxon.polypod.common.accelerators import (
    has_tpu_annotation,
    requests_gpu,
    requests_tpu,
)
from tests.utils import BaseTestCase


@pytest.mark.polypod_mark
class TestTPUs(BaseTestCase):
    def test_has_tpu_annotation(self):
        assert has_tpu_annotation(None) is False
        assert has_tpu_annotation({}) is False
        assert has_tpu_annotation({"foo": "bar"}) is False
        assert has_tpu_annotation({"tf-version.cloud-tpus.google.com": "1.13"}) is True

    def test_requests_tpu(self):
        assert (
            requests_tpu(k8s_schemas.V1ResourceRequirements(limits={"cpu": 1})) is False
        )
        assert (
            requests_tpu(
                k8s_schemas.V1ResourceRequirements(
                    limits={"cloud-tpus.google.com/v2": 1}
                )
            )
            is True
        )
        assert (
            requests_tpu(
                k8s_schemas.V1ResourceRequirements(
                    requests={"cloud-tpus.google.com/v2:": 32}
                )
            )
            is True
        )

    def test_requests_gpu(self):
        assert (
            requests_gpu(k8s_schemas.V1ResourceRequirements(limits={"cpu": 1})) is False
        )
        assert (
            requests_gpu(k8s_schemas.V1ResourceRequirements(limits={"amd.com/gpu": 1}))
            is True
        )
        assert (
            requests_gpu(
                k8s_schemas.V1ResourceRequirements(requests={"nvidia.com/gpu": 1})
            )
            is True
        )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon_sdk import V1ConnectionKind

from polyaxon.connections.reader import get_connection_type
from polyaxon.schemas.types import V1ConnectionType


def get_connection_from_type(connection_type: V1ConnectionType, **kwargs):
    # We assume that `None` refers to local store as well
    if not connection_type or connection_type.kind in {
        V1ConnectionKind.VOLUME_CLAIM,
        V1ConnectionKind.HOST_PATH,
    }:
        from polyaxon.stores.local_store import LocalStore

        return LocalStore()
    if connection_type.kind == V1ConnectionKind.WASB:
        from polyaxon.connections.azure.azure_blobstore import AzureBlobStoreService

        return AzureBlobStoreService(connection_name=connection_type.name, **kwargs)
    if connection_type.kind == V1ConnectionKind.S3:
        from polyaxon.connections.aws.s3 import S3Service

        return S3Service(connection_name=connection_type.name, **kwargs)
    if connection_type.kind == V1ConnectionKind.GCS:
        from polyaxon.connections.gcp.gcs import GCSService

        return GCSService(connection_name=connection_type.name, **kwargs)


def get_connection_from_name(connection_name: str, **kwargs):
    connection_type = get_connection_type(connection_name)
    return get_connection_from_type(connection_type=connection_type, **kwargs)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon import settings


def get_dns_config(dns_prefix=None, dns_backend=None, dns_cluster=None):
    dns_prefix = dns_prefix or settings.PROXIES_CONFIG.dns_prefix
    dns_backend = dns_backend or settings.PROXIES_CONFIG.dns_backend
    dns_cluster = dns_cluster or settings.PROXIES_CONFIG.dns_custom_cluster
    if not dns_prefix:
        dns_prefix = "{}.kube-system".format(dns_backend)
    return "{dns_prefix}.svc.{dns_cluster}".format(
        dns_prefix=dns_prefix, dns_cluster=dns_cluster
    )


def get_resolver():
    if settings.PROXIES_CONFIG.dns_use_resolver:
        dns_config = get_dns_config()
        return "resolver {} valid=5s;".format(dns_config)
    return ""



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import uuid

from typing import Optional

from polyaxon.containers.contexts import CONTEXT_MOUNT_DOCKER
from polyaxon.k8s import k8s_schemas
from polyaxon.polypod.common import constants
from polyaxon.schemas.types import V1ConnectionType, V1K8sResourceType


def get_volume_name(path: str) -> str:
    name = uuid.uuid5(namespace=uuid.NAMESPACE_DNS, name=path).hex
    return constants.CONTEXT_VOLUME_CONNECTIONS_FORMAT.format(name)


def get_volume_from_connection(
    connection: V1ConnectionType,
) -> Optional[k8s_schemas.V1Volume]:
    if not connection:
        return None
    if connection.is_volume_claim:
        pv_claim = k8s_schemas.V1PersistentVolumeClaimVolumeSource(
            claim_name=connection.schema.volume_claim,
            read_only=connection.schema.read_only,
        )
        return k8s_schemas.V1Volume(
            name=connection.name, persistent_volume_claim=pv_claim
        )

    if connection.is_host_path:
        return k8s_schemas.V1Volume(
            name=connection.name,
            host_path=k8s_schemas.V1HostPathVolumeSource(
                path=connection.schema.host_path
            ),
        )


def get_volume_from_secret(secret: V1K8sResourceType) -> Optional[k8s_schemas.V1Volume]:
    if not secret:
        return None
    if secret.schema.mount_path:
        secret_volume = k8s_schemas.V1SecretVolumeSource(
            secret_name=secret.name,
            items=secret.schema.items,
            default_mode=secret.schema.default_mode,
        )
        return k8s_schemas.V1Volume(name=secret.name, secret=secret_volume)


def get_volume_from_config_map(
    config_map: V1K8sResourceType,
) -> Optional[k8s_schemas.V1Volume]:
    if not config_map:
        return None
    if config_map.schema.mount_path:
        config_map_volume = k8s_schemas.V1ConfigMapVolumeSource(
            name=config_map.name,
            items=config_map.schema.items,
            default_mode=config_map.schema.default_mode,
        )
        return k8s_schemas.V1Volume(name=config_map.name, config_map=config_map_volume)


def get_volume(
    volume: str, claim_name: str = None, host_path: str = None, read_only: bool = None
) -> k8s_schemas.V1Volume:
    if claim_name:
        pv_claim = k8s_schemas.V1PersistentVolumeClaimVolumeSource(
            claim_name=claim_name, read_only=read_only
        )
        return k8s_schemas.V1Volume(name=volume, persistent_volume_claim=pv_claim)

    if host_path:
        return k8s_schemas.V1Volume(
            name=volume, host_path=k8s_schemas.V1HostPathVolumeSource(path=host_path)
        )

    empty_dir = k8s_schemas.V1EmptyDirVolumeSource()
    return k8s_schemas.V1Volume(name=volume, empty_dir=empty_dir)


def get_docker_context_volume() -> k8s_schemas.V1Volume:
    return get_volume(
        volume=constants.CONTEXT_VOLUME_DOCKER, host_path=CONTEXT_MOUNT_DOCKER
    )


def get_configs_context_volume() -> k8s_schemas.V1Volume:
    return get_volume(volume=constants.CONTEXT_VOLUME_CONFIGS)


def get_artifacts_context_volume() -> k8s_schemas.V1Volume:
    return get_volume(volume=constants.CONTEXT_VOLUME_ARTIFACTS)


def get_connections_context_volume(name: str) -> k8s_schemas.V1Volume:
    return get_volume(volume=name)


def get_shm_context_volume() -> k8s_schemas.V1Volume:
    """
    Mount an tmpfs volume to /dev/shm.
    This will set /dev/shm size to half of the RAM of node.
    By default, /dev/shm is very small, only 64MB.
    Some experiments will fail due to lack of share memory,
    such as some experiments running on Pytorch.
    """
    return k8s_schemas.V1Volume(
        name=constants.CONTEXT_VOLUME_SHM,
        empty_dir=k8s_schemas.V1EmptyDirVolumeSource(medium="Memory"),
    )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import fields, validate

from polyaxon.polyflow.actions import ActionSchema
from polyaxon.polyflow.cache import CacheSchema
from polyaxon.polyflow.hooks import HookSchema
from polyaxon.polyflow.plugins import PluginsSchema
from polyaxon.polyflow.termination import TerminationSchema
from polyaxon.schemas.base import NAME_REGEX, BaseCamelSchema, BaseConfig
from polyaxon.schemas.fields.ref_or_obj import RefOrObject


class BaseComponentSchema(BaseCamelSchema):
    version = fields.Float(allow_none=True)
    kind = fields.Str(allow_none=True)
    name = fields.Str(validate=validate.Regexp(regex=NAME_REGEX), allow_none=True)
    description = fields.Str(allow_none=True)
    tags = fields.List(fields.Str(), allow_none=True)
    presets = fields.List(fields.Str(allow_none=True))
    queue = RefOrObject(fields.Str(allow_none=True))
    cache = fields.Nested(CacheSchema, allow_none=True)
    termination = fields.Nested(TerminationSchema, allow_none=True)
    plugins = fields.Nested(PluginsSchema, allow_none=True)
    actions = fields.List(fields.Nested(ActionSchema), allow_none=True)
    hooks = fields.List(fields.Nested(HookSchema), allow_none=True)
    is_approved = fields.Bool(allow_none=True)

    @staticmethod
    def schema_config():
        return BaseComponent


class BaseComponent(BaseConfig):
    SCHEMA = BaseComponentSchema
    REDUCED_ATTRIBUTES = [
        "version",
        "kind",
        "name",
        "description",
        "tags",
        "presets",
        "queue",
        "cache",
        "termination",
        "plugins",
        "actions",
        "hooks",
        "isApproved",
    ]



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import INCLUDE, fields

from polyaxon.schemas.base import BaseCamelSchema, BaseConfig


class IngressSchema(BaseCamelSchema):
    enabled = fields.Bool(allow_none=True)
    host_name = fields.Str(allow_none=True)
    path = fields.Str(allow_none=True)
    tls = fields.List(fields.Dict(allow_none=True), allow_none=True)
    annotations = fields.Dict(allow_none=True)

    class Meta:
        unknown = INCLUDE

    @staticmethod
    def schema_config():
        return IngressConfig


class IngressConfig(BaseConfig):
    SCHEMA = IngressSchema
    REDUCED_ATTRIBUTES = ["enabled", "hostName", "tls", "annotations", "path"]

    def __init__(
        self,
        enabled=None,
        host_name=None,
        path=None,
        tls=None,
        annotations=None,
        **kwargs,
    ):
        self.enabled = enabled
        self.host_name = host_name
        self.path = path
        self.tls = tls
        self.annotations = annotations



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import functools

from polyaxon import settings


def check_no_op(f):
    """
    The `NoOpDecorator` is a decorator to ignore any decorated function when NO_OP env var is found.

    usage example with class method:
        @check_no_op
        def my_func(self, *args, **kwargs):
            ...
            return ...

    usage example with a function:
        @check_no_op
        def my_func(arg1, arg2):
            ...
            return ...
    """

    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        if settings.CLIENT_CONFIG.no_op:
            return None
        return f(*args, **kwargs)

    return wrapper



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from polyaxon.k8s.custom_resources.crd import get_custom_object
from polyaxon.lifecycle import V1Statuses
from polyaxon.polyflow import V1Notification
from polyaxon.polyflow.environment import V1Environment
from polyaxon.polyflow.termination import V1Termination
from polyaxon.polypod.custom_resources import get_tf_job_custom_resource
from tests.test_polypod.test_custom_resources.base_kubeflow import (
    BaseKubeflowCRDTestCase,
)


class TestTFJobCRD(BaseKubeflowCRDTestCase):
    def test_get_tf_job_custom_resource_with_no_workers(self):
        termination = V1Termination(max_retries=5, ttl=10, timeout=10)
        environment = V1Environment(
            labels={"foo": "bar"},
            annotations={"foo": "bar"},
            node_selector={"foo": "bar"},
            node_name="foo",
            restart_policy="never",
        )
        custom_object = {
            "tfJobSpec": {"cleanPodPolicy": "All", "replicaSpecs": {}},
            "termination": {
                "backoffLimit": termination.max_retries,
                "activeDeadlineSeconds": termination.timeout,
                "ttlSecondsAfterFinished": termination.ttl,
            },
            "collectLogs": False,
            "syncStatuses": False,
            "notifications": [],
        }
        expected_crd = get_custom_object(
            namespace="default",
            resource_name="foo",
            kind="Operation",
            api_version="core.polyaxon.com/v1",
            labels={"foo": "bar"},
            annotations={"foo": "long-foo-bar" * 300},
            custom_object=custom_object,
        )

        crd = get_tf_job_custom_resource(
            namespace="default",
            resource_name="foo",
            chief=None,
            worker=None,
            ps=None,
            evaluator=None,
            clean_pod_policy=None,
            termination=termination,
            collect_logs=False,
            sync_statuses=False,
            notifications=None,
            labels=environment.labels,
            annotations={"foo": "long-foo-bar" * 300},
        )

        assert crd == expected_crd

    def test_get_job_custom_resource(self):
        termination = V1Termination(max_retries=5, ttl=10, timeout=10)
        environment = V1Environment(
            labels={"foo": "bar"},
            annotations={"foo": "bar"},
            node_selector={"foo": "bar"},
            node_name="foo",
            restart_policy="never",
        )
        notifications = [V1Notification(connections=["test"], trigger=V1Statuses.DONE)]
        chief, chief_replica_template = self.get_replica(environment)
        worker, worker_replica_template = self.get_replica(environment)
        template_spec = {
            "cleanPodPolicy": "Running",
            "replicaSpecs": {
                "Chief": chief_replica_template,
                "Worker": worker_replica_template,
            },
        }
        custom_object = {
            "tfJobSpec": template_spec,
            "termination": {
                "backoffLimit": termination.max_retries,
                "activeDeadlineSeconds": termination.timeout,
                "ttlSecondsAfterFinished": termination.ttl,
            },
            "collectLogs": True,
            "syncStatuses": True,
            "notifications": [n.to_operator() for n in notifications],
        }

        expected_crd = get_custom_object(
            namespace="default",
            resource_name="foo",
            kind="Operation",
            api_version="core.polyaxon.com/v1",
            labels={"foo": "bar"},
            annotations={"foo": "bar"},
            custom_object=custom_object,
        )

        crd = get_tf_job_custom_resource(
            namespace="default",
            resource_name="foo",
            chief=chief,
            worker=worker,
            ps=None,
            evaluator=None,
            clean_pod_policy="Running",
            termination=termination,
            collect_logs=True,
            sync_statuses=True,
            notifications=notifications,
            labels=environment.labels,
            annotations={"foo": "bar"},
        )

        assert crd == expected_crd



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from polyaxon import settings
from polyaxon.auxiliaries import V1PolyaxonInitContainer, V1PolyaxonSidecarContainer
from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.connections.schemas import (
    V1BucketConnection,
    V1ClaimConnection,
    V1GitConnection,
    V1K8sResourceSchema,
)
from polyaxon.k8s import k8s_schemas
from polyaxon.polyflow import V1Init, V1Plugins
from polyaxon.polypod.common.mounts import get_mounts
from polyaxon.polypod.compiler.converters import PlatformJobConverter
from polyaxon.polypod.init.artifacts import get_artifacts_path_container
from polyaxon.polypod.init.auth import get_auth_context_container
from polyaxon.polypod.init.dockerfile import get_dockerfile_init_container
from polyaxon.polypod.init.git import get_git_init_container
from polyaxon.polypod.init.store import get_store_container
from polyaxon.polypod.main.container import get_main_container
from polyaxon.polypod.sidecar.container import get_sidecar_container
from polyaxon.polypod.specs.contexts import PluginsContextsSpec
from polyaxon.schemas.types import (
    V1ArtifactsType,
    V1ConnectionType,
    V1DockerfileType,
    V1K8sResourceType,
)
from polyaxon.services.headers import PolyaxonServices
from tests.utils import BaseTestCase


class DummyConverter(PlatformJobConverter):
    SPEC_KIND = "dumy"
    K8S_ANNOTATIONS_KIND = "dummy-name"
    MAIN_CONTAINER_ID = "dummy"


class TestJobConverter(BaseTestCase):
    SET_AGENT_SETTINGS = True

    def setUp(self):
        super().setUp()
        settings.AGENT_CONFIG.app_secret_name = "polyaxon"
        settings.AGENT_CONFIG.agent_secret_name = "agent"
        settings.CLIENT_CONFIG.host = "https://polyaxon.com"
        self.converter = DummyConverter(
            owner_name="owner-name",
            project_name="project-name",
            run_name="run-name",
            run_uuid="run_uuid",
        )

    def test_get_main_env_vars(self):
        env_vars = self.converter.get_main_env_vars()
        assert env_vars == self.converter.get_service_env_vars(
            service_header=PolyaxonServices.RUNNER
        )

    def test_get_init_containers_with_auth(self):
        containers = self.converter.get_init_containers(
            polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
            contexts=PluginsContextsSpec.from_config(
                V1Plugins(collect_logs=False, collect_artifacts=False, auth=True)
            ),
            artifacts_store=None,
            init_connections=None,
            connection_by_names={},
            init_containers=[],
        )
        assert containers == [
            get_auth_context_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                env=self.converter.get_auth_service_env_vars(),
            )
        ]

    def test_get_init_containers_none(self):
        containers = self.converter.get_init_containers(
            polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
            contexts=None,
            artifacts_store=None,
            init_connections=None,
            connection_by_names={},
            init_containers=[],
        )
        assert containers == []

    def test_get_init_containers_with_claim_outputs(self):
        store = V1ConnectionType(
            name="test_claim",
            kind=V1ConnectionKind.VOLUME_CLAIM,
            schema=V1ClaimConnection(
                mount_path="/claim/path", volume_claim="claim", read_only=True
            ),
        )

        # No context to enable the outputs
        containers = self.converter.get_init_containers(
            contexts=None,
            artifacts_store=store.name,
            init_connections=None,
            connection_by_names={},
            init_containers=[],
            polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
        )
        assert containers == []

        # Enable outputs
        containers = self.converter.get_init_containers(
            contexts=PluginsContextsSpec.from_config(
                V1Plugins(collect_artifacts=True, collect_logs=False)
            ),
            artifacts_store=store,
            connection_by_names={},
            init_connections=None,
            init_containers=[],
            polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
        )
        assert containers == [
            get_artifacts_path_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                artifacts_store=store,
                run_path=self.converter.run_path,
                auto_resume=True,
            ),
        ]

        # Use store for init
        containers = self.converter.get_init_containers(
            contexts=None,
            artifacts_store=None,
            connection_by_names={store.name: store},
            init_connections=[V1Init(connection=store.name)],
            init_containers=[],
            polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
        )
        assert containers == [
            get_store_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                connection=store,
                artifacts=None,
                env=self.converter.get_init_service_env_vars(),
            )
        ]

        # Use store for init and outputs
        containers = self.converter.get_init_containers(
            contexts=PluginsContextsSpec.from_config(
                V1Plugins(collect_artifacts=True, collect_logs=False)
            ),
            artifacts_store=store,
            init_connections=[V1Init(connection=store.name)],
            connection_by_names={store.name: store},
            init_containers=[],
            polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
        )
        assert containers == [
            get_artifacts_path_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                artifacts_store=store,
                run_path=self.converter.run_path,
                auto_resume=True,
            ),
            get_store_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                connection=store,
                artifacts=None,
                env=self.converter.get_init_service_env_vars(),
                is_default_artifacts_store=True,
            ),
        ]

        # Add Store
        store1 = V1ConnectionType(
            name="test_gcs",
            kind=V1ConnectionKind.S3,
            schema=V1BucketConnection(bucket="s3://foo"),
            secret=None,
        )

        containers = self.converter.get_init_containers(
            contexts=PluginsContextsSpec.from_config(
                V1Plugins(collect_artifacts=True, collect_logs=False, auth=True)
            ),
            artifacts_store=store,
            init_connections=[
                V1Init(
                    connection=store.name,
                    artifacts=V1ArtifactsType(files=["/foo", "/bar"]),
                ),
                V1Init(
                    connection=store1.name,
                    artifacts=V1ArtifactsType(files=["/foo", "/bar"]),
                ),
            ],
            connection_by_names={store.name: store, store1.name: store1},
            init_containers=[],
            polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
        )
        assert containers == [
            get_auth_context_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                env=self.converter.get_auth_service_env_vars(),
            ),
            get_artifacts_path_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                artifacts_store=store,
                run_path=self.converter.run_path,
                auto_resume=True,
            ),
            get_store_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                connection=store,
                artifacts=V1ArtifactsType(files=["/foo", "/bar"]),
                env=self.converter.get_init_service_env_vars(),
                is_default_artifacts_store=True,
            ),
            get_store_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                connection=store1,
                artifacts=V1ArtifactsType(files=["/foo", "/bar"]),
                env=self.converter.get_init_service_env_vars(),
            ),
        ]

    def test_get_init_containers_with_dockerfiles(self):
        dockerfile_args1 = V1DockerfileType(
            image="foo/test", lang_env="LANG", env=[], run=["step1", "step2"]
        )
        dockerfile_args2 = V1DockerfileType(
            image="foo/test",
            lang_env="LANG",
            env=[],
            run=["step1", "step2"],
            filename="dockerfile2",
            path="/test",
        )
        containers = self.converter.get_init_containers(
            contexts=None,
            artifacts_store=None,
            init_connections=[
                V1Init(dockerfile=dockerfile_args1),
                V1Init(dockerfile=dockerfile_args2, path="/test"),
            ],
            init_containers=[],
            connection_by_names={},
            polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
        )
        for container in containers:
            container.name = ""
        expected_containers = [
            get_dockerfile_init_container(
                dockerfile_args=dockerfile_args1,
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                env=self.converter.get_init_service_env_vars(),
                contexts=None,
                run_path=self.converter.run_path,
                run_instance=self.converter.run_instance,
            ),
            get_dockerfile_init_container(
                dockerfile_args=dockerfile_args2,
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                env=self.converter.get_init_service_env_vars(),
                mount_path="/test",
                contexts=None,
                run_path=self.converter.run_path,
                run_instance=self.converter.run_instance,
            ),
        ]
        for container in expected_containers:
            container.name = ""

        assert expected_containers == containers

    def test_get_init_containers_with_git_without_connection(self):
        git1 = V1GitConnection(revision="test", url="https://test.com")
        git2 = V1GitConnection(revision="test", url="https://test.com")
        containers = self.converter.get_init_containers(
            contexts=None,
            artifacts_store=None,
            init_connections=[
                V1Init(git=git1, container=k8s_schemas.V1Container(name="test")),
                V1Init(git=git2, path="/test"),
            ],
            init_containers=[],
            connection_by_names={},
            polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
        )
        assert containers == [
            get_git_init_container(
                connection=V1ConnectionType(
                    name=git1.get_name(), kind=V1ConnectionKind.GIT, schema=git1
                ),
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                env=self.converter.get_init_service_env_vars(),
                contexts=None,
            ),
            get_git_init_container(
                container=k8s_schemas.V1Container(name="test"),
                connection=V1ConnectionType(
                    name=git2.get_name(), kind=V1ConnectionKind.GIT, schema=git1
                ),
                mount_path="/test",
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                env=self.converter.get_init_service_env_vars(),
                contexts=None,
            ),
        ]

    def test_get_init_containers_with_store_outputs(self):
        store = V1ConnectionType(
            name="test_gcs",
            kind=V1ConnectionKind.S3,
            schema=V1BucketConnection(bucket="s3://foo"),
        )

        # No context
        containers = self.converter.get_init_containers(
            contexts=None,
            artifacts_store=store,
            init_connections=[],
            init_containers=[],
            connection_by_names={},
            polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
        )
        assert containers == []

        # With context
        containers = self.converter.get_init_containers(
            contexts=PluginsContextsSpec.from_config(
                V1Plugins(collect_artifacts=True, collect_logs=False, auth=True)
            ),
            artifacts_store=store,
            init_connections=[],
            init_containers=[],
            connection_by_names={},
            polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
        )
        assert containers == [
            get_auth_context_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                env=self.converter.get_auth_service_env_vars(),
            ),
            get_artifacts_path_container(
                polyaxon_init=V1PolyaxonInitContainer(image="foo/foo"),
                artifacts_store=store,
                run_path=self.converter.run_path,
                auto_resume=True,
            ),
        ]

    def test_get_sidecars(self):
        assert (
            self.converter.get_sidecar_containers(
                contexts=None,
                artifacts_store=None,
                sidecar_containers=[],
                polyaxon_sidecar=V1PolyaxonSidecarContainer(
                    image="sidecar/sidecar", sleep_interval=12, sync_interval=-1
                ),
            )
            == []
        )

        # Store with single path, no secret is passed and not required
        store = V1ConnectionType(
            name="test",
            kind=V1ConnectionKind.S3,
            schema=V1BucketConnection(bucket="s3://foo"),
        )
        contexts = PluginsContextsSpec.from_config(
            V1Plugins(collect_logs=True, collect_artifacts=True, auth=True)
        )
        assert self.converter.get_sidecar_containers(
            contexts=contexts,
            artifacts_store=store,
            sidecar_containers=[],
            polyaxon_sidecar=V1PolyaxonSidecarContainer(
                image="sidecar/sidecar", sleep_interval=12, sync_interval=12
            ),
        ) == [
            get_sidecar_container(
                container_id="dummy",
                contexts=contexts,
                env=self.converter.get_polyaxon_sidecar_service_env_vars(),
                polyaxon_sidecar=V1PolyaxonSidecarContainer(
                    image="sidecar/sidecar", sleep_interval=12, sync_interval=12
                ),
                artifacts_store=store,
                run_path=self.converter.run_path,
            )
        ]

        secret1 = V1K8sResourceType(
            name="test1",
            schema=V1K8sResourceSchema(name="ref1", items=["item1", "item2"]),
            is_requested=True,
        )
        store.secret = secret1.schema

        polyaxon_sidecar = V1PolyaxonSidecarContainer(
            image="sidecar/sidecar",
            image_pull_policy=None,
            sleep_interval=12,
            sync_interval=-1,
        )

        assert self.converter.get_sidecar_containers(
            contexts=contexts,
            artifacts_store=store,
            polyaxon_sidecar=polyaxon_sidecar,
            sidecar_containers=[],
        ) == [
            get_sidecar_container(
                container_id="dummy",
                contexts=contexts,
                env=self.converter.get_polyaxon_sidecar_service_env_vars(),
                polyaxon_sidecar=polyaxon_sidecar,
                artifacts_store=store,
                run_path=self.converter.run_path,
            )
        ]

    def test_main_container(self):
        store = V1ConnectionType(
            name="test_gcs",
            kind=V1ConnectionKind.S3,
            schema=V1BucketConnection(bucket="s3://foo"),
            secret=None,
        )
        contexts = PluginsContextsSpec.from_config(
            V1Plugins.from_dict({}), default_auth=True
        )
        main_container = k8s_schemas.V1Container(
            name="main",
            image="foo/test",
            image_pull_policy="IfNotPresent",
            command=["foo", "bar"],
            args=["arg1", "arg2"],
        )
        container = self.converter.get_main_container(
            main_container=main_container,
            contexts=contexts,
            artifacts_store=store,
            init_connections=[],
            connections=[],
            connection_by_names={},
            log_level="info",
            secrets=[],
            config_maps=[],
            kv_env_vars=[],
            ports=None,
        )
        expected_container = get_main_container(
            container_id="dummy",
            main_container=main_container,
            contexts=contexts,
            volume_mounts=get_mounts(
                use_auth_context=True,
                use_artifacts_context=False,
                use_docker_context=False,
                use_shm_context=False,
            ),
            log_level="info",
            artifacts_store=store,
            connections=[],
            init=[],
            connection_by_names={},
            secrets=[],
            config_maps=[],
            kv_env_vars=[],
            env=self.converter.get_main_env_vars(),
            ports=None,
            run_path="/test",
        )

        assert container == expected_container



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

import click

from polyaxon import settings
from polyaxon.utils.http_utils import clean_host


def get_dashboard_url(base: str = "ui", subpath: str = "") -> str:
    dashboard_url = "{}/{}/".format(clean_host(settings.CLIENT_CONFIG.host), base)
    if subpath:
        return "{}{}/".format(dashboard_url, subpath.rstrip("/"))
    return dashboard_url


def get_dashboard(dashboard_url: str, url_only: bool, yes: bool):
    if url_only:
        click.echo(dashboard_url)
        sys.exit(0)
    if not yes:
        click.confirm(
            "Dashboard page will now open in your browser. Continue?",
            abort=True,
            default=True,
        )

    click.launch(dashboard_url)


@click.command()
@click.option(
    "--yes",
    "-y",
    is_flag=True,
    default=False,
    help="Automatic yes to prompts. "
    'Assume "yes" as answer to all prompts and run non-interactively.',
)
@click.option(
    "--url", is_flag=True, default=False, help="Print the url of the dashboard."
)
def dashboard(yes, url):
    """Open dashboard in browser."""
    get_dashboard(dashboard_url=get_dashboard_url(), url_only=url, yes=yes)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from polyaxon_sdk import V1BucketConnection

from polyaxon.connections.kinds import V1ConnectionKind
from polyaxon.connections.schemas import (
    V1ClaimConnection,
    V1HostPathConnection,
    V1K8sResourceSchema,
)
from polyaxon.polyflow import V1Init, V1Plugins
from polyaxon.polypod.common.mounts import (
    get_artifacts_context_mount,
    get_connections_context_mount,
    get_mount_from_resource,
    get_mount_from_store,
)
from polyaxon.polypod.common.volumes import get_volume, get_volume_name
from polyaxon.polypod.main.volumes import get_volume_mounts
from polyaxon.polypod.specs.contexts import PluginsContextsSpec
from polyaxon.schemas.types import V1ConnectionType, V1K8sResourceType
from tests.utils import BaseTestCase


@pytest.mark.polypod_mark
class TestMainMounts(BaseTestCase):
    def setUp(self):
        super().setUp()
        # Secrets and config maps
        self.non_mount_resource1 = V1K8sResourceType(
            name="non_mount_test1",
            schema=V1K8sResourceSchema(name="ref", items=["item1", "item2"]),
            is_requested=False,
        )
        self.non_mount_resource2 = V1K8sResourceType(
            name="non_mount_test2",
            schema=V1K8sResourceSchema(name="ref"),
            is_requested=False,
        )
        self.mount_resource1 = V1K8sResourceType(
            name="mount_test1",
            schema=V1K8sResourceSchema(
                name="ref", items=["item1", "item2"], mount_path="/tmp1"
            ),
            is_requested=False,
        )
        self.mount_resource2 = V1K8sResourceType(
            name="mount_test2",
            schema=V1K8sResourceSchema(name="ref", mount_path="/tmp2"),
            is_requested=False,
        )
        # Volumes
        self.vol1 = get_volume(volume="vol1", claim_name="claim1")
        self.vol2 = get_volume(volume="vol2", host_path="/path2")
        self.vol3 = get_volume(volume="vol3")
        # connections
        self.s3_store = V1ConnectionType(
            name="test_s3",
            kind=V1ConnectionKind.S3,
            schema=V1BucketConnection(bucket="s3//:foo"),
            secret=self.mount_resource1.schema,
        )
        self.gcs_store = V1ConnectionType(
            name="test_gcs",
            kind=V1ConnectionKind.GCS,
            schema=V1BucketConnection(bucket="gs//:foo"),
            secret=self.mount_resource1.schema,
        )
        self.az_store = V1ConnectionType(
            name="test_az",
            kind=V1ConnectionKind.WASB,
            schema=V1BucketConnection(bucket="wasb://x@y.blob.core.windows.net"),
            secret=self.mount_resource1.schema,
        )
        self.claim_store = V1ConnectionType(
            name="test_claim",
            kind=V1ConnectionKind.VOLUME_CLAIM,
            schema=V1ClaimConnection(
                mount_path="/tmp", volume_claim="test", read_only=True
            ),
        )
        self.host_path_store = V1ConnectionType(
            name="test_path",
            kind=V1ConnectionKind.HOST_PATH,
            schema=V1HostPathConnection(
                mount_path="/tmp", host_path="/tmp", read_only=True
            ),
        )

    def test_get_volume_mounts(self):
        assert (
            get_volume_mounts(
                contexts=None,
                init=None,
                connections=None,
                secrets=None,
                config_maps=None,
            )
            == []
        )
        assert (
            get_volume_mounts(
                contexts=None, init=[], connections=[], secrets=[], config_maps=[]
            )
            == []
        )

    @staticmethod
    def assert_contexts_store(contexts, results):
        assert (
            get_volume_mounts(
                contexts=contexts, init=[], connections=[], secrets=[], config_maps=[]
            )
            == results
        )

    def test_artifacts_store(self):
        self.assert_contexts_store(contexts=None, results=[])
        self.assert_contexts_store(
            contexts=PluginsContextsSpec.from_config(
                PluginsContextsSpec.from_config(
                    V1Plugins(collect_logs=False, collect_artifacts=True)
                )
            ),
            results=[get_artifacts_context_mount(read_only=False)],
        )
        self.assert_contexts_store(
            contexts=PluginsContextsSpec.from_config(
                PluginsContextsSpec.from_config(
                    V1Plugins(collect_logs=True, collect_artifacts=False)
                )
            ),
            results=[],
        )
        self.assert_contexts_store(
            contexts=PluginsContextsSpec.from_config(
                PluginsContextsSpec.from_config(
                    V1Plugins(collect_logs=True, collect_artifacts=True)
                )
            ),
            results=[get_artifacts_context_mount(read_only=False)],
        )

    @staticmethod
    def assert_single_store(store, results):
        assert (
            get_volume_mounts(
                contexts=None, init=[], connections=[store], secrets=[], config_maps=[]
            )
            == results
        )

    @staticmethod
    def assert_single_init_store(store, results):
        assert (
            get_volume_mounts(
                contexts=None,
                init=[V1Init(connection=store.name, path="/test")],
                connections=[],
                secrets=[],
                config_maps=[],
            )
            == results
        )

    def test_single_connections(self):
        self.assert_single_store(store=self.s3_store, results=[])
        self.assert_single_store(store=self.gcs_store, results=[])
        self.assert_single_store(store=self.az_store, results=[])
        self.assert_single_store(
            store=self.claim_store,
            results=[get_mount_from_store(store=self.claim_store)],
        )
        self.assert_single_store(
            store=self.host_path_store,
            results=[get_mount_from_store(store=self.host_path_store)],
        )

        # Managed versions
        volume_name = get_volume_name("/test")
        self.assert_single_init_store(
            store=self.s3_store,
            results=[
                get_connections_context_mount(name=volume_name, mount_path="/test")
            ],
        )
        self.assert_single_init_store(
            store=self.gcs_store,
            results=[
                get_connections_context_mount(name=volume_name, mount_path="/test")
            ],
        )
        self.assert_single_init_store(
            store=self.az_store,
            results=[
                get_connections_context_mount(name=volume_name, mount_path="/test")
            ],
        )
        self.assert_single_init_store(
            store=self.claim_store,
            results=[
                get_connections_context_mount(name=volume_name, mount_path="/test")
            ],
        )
        self.assert_single_init_store(
            store=self.host_path_store,
            results=[
                get_connections_context_mount(name=volume_name, mount_path="/test")
            ],
        )

    def test_multi_connections(self):
        assert (
            len(
                get_volume_mounts(
                    contexts=None,
                    init=[],
                    connections=[
                        self.s3_store,
                        self.gcs_store,
                        self.az_store,
                        self.claim_store,
                        self.host_path_store,
                    ],
                    secrets=[],
                    config_maps=[],
                )
            )
            == 2
        )

        assert (
            len(
                get_volume_mounts(
                    contexts=None,
                    init=[
                        V1Init(connection=self.s3_store.name, path="/test-1"),
                        V1Init(connection=self.gcs_store.name, path="/test-2"),
                        V1Init(connection=self.az_store.name, path="/test-3"),
                        V1Init(connection=self.claim_store.name, path="/test-4"),
                        V1Init(connection=self.host_path_store.name, path="/test-5"),
                    ],
                    connections=[],
                    secrets=[],
                    config_maps=[],
                )
            )
            == 5
        )

        assert (
            len(
                get_volume_mounts(
                    contexts=None,
                    init=[
                        V1Init(connection=self.s3_store.name, path="/test-1"),
                        V1Init(connection=self.gcs_store.name, path="/test-2"),
                        V1Init(connection=self.az_store.name, path="/test-3"),
                        V1Init(connection=self.claim_store.name, path="/test-4"),
                        V1Init(connection=self.host_path_store.name, path="/test-5"),
                    ],
                    connections=[
                        self.s3_store,
                        self.gcs_store,
                        self.az_store,
                        self.claim_store,
                        self.host_path_store,
                    ],
                    secrets=[],
                    config_maps=[],
                )
            )
            == 7
        )

    @staticmethod
    def assert_secret(secret, results):
        assert (
            get_volume_mounts(
                contexts=None, init=[], connections=[], secrets=[secret], config_maps=[]
            )
            == results
        )

    @staticmethod
    def assert_config_map(config_map, results):
        assert (
            get_volume_mounts(
                contexts=None,
                init=[],
                connections=[],
                secrets=[],
                config_maps=[config_map],
            )
            == results
        )

    def test_secret_volumes(self):
        self.assert_secret(secret=self.non_mount_resource1, results=[])
        self.assert_secret(secret=self.non_mount_resource2, results=[])
        self.assert_secret(
            secret=self.mount_resource1,
            results=[get_mount_from_resource(resource=self.mount_resource1)],
        )
        self.assert_secret(
            secret=self.mount_resource2,
            results=[get_mount_from_resource(resource=self.mount_resource2)],
        )

    def test_config_map_volumes(self):
        self.assert_config_map(config_map=self.non_mount_resource1, results=[])
        self.assert_config_map(config_map=self.non_mount_resource2, results=[])
        self.assert_config_map(
            config_map=self.mount_resource1,
            results=[get_mount_from_resource(resource=self.mount_resource1)],
        )
        self.assert_config_map(
            config_map=self.mount_resource2,
            results=[get_mount_from_resource(resource=self.mount_resource2)],
        )

    def test_multiple_resources(self):
        assert get_volume_mounts(
            contexts=None,
            init=[],
            connections=[],
            secrets=[
                self.non_mount_resource1,
                self.non_mount_resource1,
                self.mount_resource1,
                self.mount_resource2,
            ],
            config_maps=[
                self.non_mount_resource1,
                self.non_mount_resource1,
                self.mount_resource1,
                self.mount_resource2,
            ],
        ) == [
            get_mount_from_resource(resource=self.mount_resource1),
            get_mount_from_resource(resource=self.mount_resource2),
            get_mount_from_resource(resource=self.mount_resource1),
            get_mount_from_resource(resource=self.mount_resource2),
        ]

    def test_all_volumes(self):
        assert (
            len(
                get_volume_mounts(
                    contexts=PluginsContextsSpec.from_config(
                        V1Plugins(collect_logs=False, collect_artifacts=True)
                    ),
                    init=[
                        V1Init(connection=self.s3_store.name, path="/test-1"),
                        V1Init(connection=self.gcs_store.name, path="/test-2"),
                        V1Init(connection=self.az_store.name, path="/test-3"),
                        V1Init(connection=self.claim_store.name, path="/test-4"),
                        V1Init(connection=self.host_path_store.name, path="/test-5"),
                    ],
                    connections=[
                        self.s3_store,
                        self.gcs_store,
                        self.az_store,
                        self.claim_store,
                        self.host_path_store,
                    ],
                    secrets=[
                        self.non_mount_resource1,
                        self.non_mount_resource1,
                        self.mount_resource1,
                        self.mount_resource2,
                    ],
                    config_maps=[
                        self.non_mount_resource1,
                        self.non_mount_resource1,
                        self.mount_resource1,
                        self.mount_resource2,
                    ],
                )
            )
            # 1: output store
            # 7: 5 managed contexts + 2 mounts
            # 4: 4 mount resources (secrets + configs)
            == 1 + 7 + 4
        )
        assert (
            len(
                get_volume_mounts(
                    contexts=None,
                    init=[
                        V1Init(connection=self.s3_store.name, path="/test-1"),
                        V1Init(connection=self.gcs_store.name, path="/test-2"),
                        V1Init(connection=self.az_store.name, path="/test-3"),
                        V1Init(connection=self.claim_store.name, path="/test-4"),
                        V1Init(connection=self.host_path_store.name, path="/test-5"),
                    ],
                    connections=[
                        self.s3_store,
                        self.gcs_store,
                        self.az_store,
                        self.claim_store,
                        self.host_path_store,
                    ],
                    secrets=[
                        self.non_mount_resource1,
                        self.non_mount_resource1,
                        self.mount_resource1,
                        self.mount_resource2,
                    ],
                    config_maps=[
                        self.non_mount_resource1,
                        self.non_mount_resource1,
                        self.mount_resource1,
                        self.mount_resource2,
                    ],
                )
            )
            # 7: 5 managed contexts + 2 mounts
            # 4: 4 mount resources (secrets + configs)
            == 7 + 4
        )

        assert (
            len(
                get_volume_mounts(
                    contexts=PluginsContextsSpec.from_config(
                        V1Plugins(collect_logs=True, collect_artifacts=True)
                    ),
                    init=[
                        V1Init(connection=self.s3_store.name, path="/test-1"),
                        V1Init(connection=self.gcs_store.name, path="/test-2"),
                        V1Init(connection=self.az_store.name, path="/test-3"),
                        V1Init(connection=self.claim_store.name, path="/test-4"),
                        V1Init(connection=self.host_path_store.name, path="/test-5"),
                    ],
                    connections=[
                        self.s3_store,
                        self.gcs_store,
                        self.az_store,
                        self.claim_store,
                        self.host_path_store,
                    ],
                    secrets=[
                        self.non_mount_resource1,
                        self.non_mount_resource1,
                        self.mount_resource1,
                        self.mount_resource2,
                    ],
                    config_maps=[
                        self.non_mount_resource1,
                        self.non_mount_resource1,
                        self.mount_resource1,
                        self.mount_resource2,
                    ],
                )
            )
            # 1: outputs context store
            # 7: 5 managed contexts + 2 mounts
            # 4: 4 mount resources (secrets + configs)
            == 1 + 7 + 4
        )
        assert (
            len(
                get_volume_mounts(
                    contexts=PluginsContextsSpec.from_config(
                        V1Plugins(collect_logs=True, collect_artifacts=False)
                    ),
                    init=[
                        V1Init(connection=self.s3_store.name, path="/test-1"),
                        V1Init(connection=self.gcs_store.name, path="/test-2"),
                        V1Init(connection=self.az_store.name, path="/test-3"),
                        V1Init(connection=self.claim_store.name, path="/test-4"),
                        V1Init(connection=self.host_path_store.name, path="/test-5"),
                    ],
                    connections=[
                        self.s3_store,
                        self.gcs_store,
                        self.az_store,
                        self.claim_store,
                        self.host_path_store,
                    ],
                    secrets=[
                        self.non_mount_resource1,
                        self.non_mount_resource1,
                        self.mount_resource1,
                        self.mount_resource2,
                    ],
                    config_maps=[
                        self.non_mount_resource1,
                        self.non_mount_resource1,
                        self.mount_resource1,
                        self.mount_resource2,
                    ],
                )
            )
            # 7: 5 managed contexts + 2 mounts
            # 4: 4 mount resources (secrets + configs)
            == 7 + 4
        )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum

from polyaxon.utils.enums_utils import enum_to_choices, enum_to_set, values_to_choices
from tests.utils import BaseTestCase


class Dummy1Enum(Enum):
    A = 1
    B = 2


class Dummy2Enum(Enum):
    A = "A"
    B = "B"


class TestEnums(BaseTestCase):
    def test_enum_to_choices(self):
        assert enum_to_choices(Dummy1Enum) == ((1, 1), (2, 2))
        assert enum_to_choices(Dummy2Enum) == (("A", "A"), ("B", "B"))

    def test_values_to_choices(self):
        assert values_to_choices({1, 2}) == ((1, 1), (2, 2))
        assert values_to_choices(["A", "B"]) == (("A", "A"), ("B", "B"))

    def test_enum_to_set(self):
        assert enum_to_set(Dummy1Enum) == {1, 2}
        assert enum_to_set(Dummy2Enum) == {"A", "B"}



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from typing import List, Optional, Union

import boto3

from polyaxon.connections.base import BaseService
from polyaxon.connections.reader import get_connection_context_path, read_keys
from polyaxon.exceptions import PolyaxonConnectionError


def get_aws_access_key_id(
    keys: Optional[Union[str, List[str]]] = None, context_path: Optional[str] = None
):
    keys = keys or ["AWS_ACCESS_KEY_ID"]
    return read_keys(context_path=context_path, keys=keys)


def get_aws_secret_access_key(
    keys: Optional[Union[str, List[str]]] = None, context_path: Optional[str] = None
):
    keys = keys or ["AWS_SECRET_ACCESS_KEY"]
    return read_keys(context_path=context_path, keys=keys)


def get_aws_security_token(
    keys: Optional[Union[str, List[str]]] = None, context_path: Optional[str] = None
):
    keys = keys or ["AWS_SECURITY_TOKEN"]
    return read_keys(context_path=context_path, keys=keys)


def get_region(
    keys: Optional[Union[str, List[str]]] = None, context_path: Optional[str] = None
):
    keys = keys or ["AWS_REGION"]
    return read_keys(context_path=context_path, keys=keys)


def get_endpoint_url(
    keys: Optional[Union[str, List[str]]] = None, context_path: Optional[str] = None
):
    keys = keys or ["AWS_ENDPOINT_URL"]
    return read_keys(context_path=context_path, keys=keys)


def get_aws_use_ssl(
    keys: Optional[Union[str, List[str]]] = None, context_path: Optional[str] = None
):
    keys = keys or ["AWS_USE_SSL"]
    return read_keys(context_path=context_path, keys=keys)


def get_aws_verify_ssl(
    keys: Optional[Union[str, List[str]]] = None, context_path: Optional[str] = None
):
    keys = keys or ["AWS_VERIFY_SSL"]
    return read_keys(context_path=context_path, keys=keys)


def get_aws_legacy_api(
    keys: Optional[Union[str, List[str]]] = None, context_path: Optional[str] = None
):
    keys = keys or ["AWS_LEGACY_API"]
    return read_keys(context_path=context_path, keys=keys)


def get_legacy_api(legacy_api=False):
    legacy_api = legacy_api or get_aws_legacy_api()
    return legacy_api


def get_aws_session(
    aws_access_key_id=None,
    aws_secret_access_key=None,
    aws_session_token=None,
    region_name=None,
    context_path=None,
):
    aws_access_key_id = aws_access_key_id or get_aws_access_key_id(
        context_path=context_path
    )
    aws_secret_access_key = aws_secret_access_key or get_aws_secret_access_key(
        context_path=context_path
    )
    aws_session_token = aws_session_token or get_aws_security_token(
        context_path=context_path
    )
    region_name = region_name or get_region(context_path=context_path)
    return boto3.session.Session(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        aws_session_token=aws_session_token,
        region_name=region_name,
    )


def get_aws_client(
    client_type,
    endpoint_url=None,
    aws_access_key_id=None,
    aws_secret_access_key=None,
    aws_session_token=None,
    region_name=None,
    aws_use_ssl=True,
    aws_verify_ssl=None,
    context_path=None,
):
    session = get_aws_session(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        aws_session_token=aws_session_token,
        region_name=region_name,
        context_path=context_path,
    )
    endpoint_url = endpoint_url or get_endpoint_url(context_path=context_path)
    aws_use_ssl = aws_use_ssl or get_aws_use_ssl(context_path=context_path)
    if aws_verify_ssl is None:
        aws_verify_ssl = get_aws_verify_ssl(context_path=context_path)
    else:
        aws_verify_ssl = aws_verify_ssl
    return session.client(
        client_type,
        endpoint_url=endpoint_url,
        use_ssl=aws_use_ssl,
        verify=aws_verify_ssl,
    )


def get_aws_resource(
    resource_type,
    endpoint_url=None,
    aws_access_key_id=None,
    aws_secret_access_key=None,
    aws_session_token=None,
    region_name=None,
    aws_use_ssl=True,
    aws_verify_ssl=None,
    context_path=None,
):
    session = get_aws_session(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        aws_session_token=aws_session_token,
        region_name=region_name,
        context_path=context_path,
    )
    endpoint_url = endpoint_url or get_endpoint_url(context_path=context_path)
    aws_use_ssl = aws_use_ssl or get_aws_use_ssl(context_path=context_path)
    if aws_verify_ssl is None:
        aws_verify_ssl = get_aws_verify_ssl(context_path=context_path)
    else:
        aws_verify_ssl = aws_verify_ssl
    return session.resource(
        resource_type,
        endpoint_url=endpoint_url,
        use_ssl=aws_use_ssl,
        verify=aws_verify_ssl,
    )


class AWSService(BaseService):

    ENCRYPTION = "AES256"
    RESOURCE_TYPE = ""

    def __init__(self, connection=None, resource=None, **kwargs):
        super().__init__(connection=connection, **kwargs)
        if not self.RESOURCE_TYPE:
            raise PolyaxonConnectionError("Aws connection requires a RESOURCE_TYPE")
        self._resource = resource
        self._encoding = kwargs.get("encoding", "utf-8")
        self._endpoint_url = (
            kwargs.get("endpoint_url")
            or kwargs.get("aws_endpoint_url")
            or kwargs.get("AWS_ENDPOINT_URL")
        )
        self._aws_access_key_id = (
            kwargs.get("access_key_id")
            or kwargs.get("aws_access_key_id")
            or kwargs.get("AWS_ACCESS_KEY_ID")
        )
        self._aws_secret_access_key = (
            kwargs.get("secret_access_key")
            or kwargs.get("aws_secret_access_key")
            or kwargs.get("AWS_SECRET_ACCESS_KEY")
        )
        self._aws_session_token = (
            kwargs.get("session_token")
            or kwargs.get("aws_session_token")
            or kwargs.get("AWS_SECURITY_TOKEN")
        )
        self._region_name = (
            kwargs.get("region") or kwargs.get("aws_region") or kwargs.get("AWS_REGION")
        )
        self._aws_verify_ssl = kwargs.get(
            "verify_ssl",
            kwargs.get("aws_verify_ssl", kwargs.get("AWS_VERIFY_SSL", None)),
        )
        self._aws_use_ssl = (
            kwargs.get("use_ssl")
            or kwargs.get("aws_use_ssl")
            or kwargs.get("AWS_USE_SSL")
        )
        self._aws_legacy_api = (
            kwargs.get("legacy_api")
            or kwargs.get("aws_legacy_api")
            or kwargs.get("AWS_LEGACY_API")
        )

    def set_connection(
        self,
        connection=None,
        connection_type=None,
        endpoint_url=None,
        aws_access_key_id=None,
        aws_secret_access_key=None,
        aws_session_token=None,
        region_name=None,
        aws_use_ssl=True,
        aws_verify_ssl=None,
    ):
        """
        Sets a new connection.

        Args:
            endpoint_url: `str`. The complete URL to use for the constructed client.
            aws_access_key_id: `str`. The access key to use when creating the client.
            aws_secret_access_key: `str`. The secret key to use when creating the client.
            aws_session_token: `str`. The session token to use when creating the client.
            region_name: `str`. The name of the region associated with the client.
                A client is associated with a single region.

        Returns:
            Service client instance
        """
        if connection:
            self._connection = connection
            return
        connection_type = connection_type or self._connection_type
        connection_name = connection_type.name if connection_type else None
        context_path = get_connection_context_path(name=connection_name)
        self._connection = get_aws_client(
            self.RESOURCE_TYPE or self.resource,
            endpoint_url=endpoint_url,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token,
            region_name=region_name,
            aws_use_ssl=aws_use_ssl,
            aws_verify_ssl=aws_verify_ssl,
            context_path=context_path,
        )

    def set_env_vars(self):
        if self._endpoint_url:
            os.environ["AWS_ENDPOINT_URL"] = self._endpoint_url
        if self._aws_access_key_id:
            os.environ["AWS_ACCESS_KEY_ID"] = self._aws_access_key_id
        if self._aws_secret_access_key:
            os.environ["AWS_SECRET_ACCESS_KEY"] = self._aws_secret_access_key
        if self._aws_session_token:
            os.environ["AWS_SECURITY_TOKEN"] = self._aws_session_token
        if self._region_name:
            os.environ["AWS_REGION"] = self._region_name
        if self._aws_use_ssl is not None:
            os.environ["AWS_USE_SSL"] = self._aws_use_ssl
        if self._aws_verify_ssl is not None:
            os.environ["AWS_VERIFY_SSL"] = self._aws_verify_ssl
        if self._aws_legacy_api:
            os.environ["AWS_LEGACY_API"] = self._aws_legacy_api

    @property
    def resource(self):
        if self._resource is None:
            self.set_resource(
                endpoint_url=self._endpoint_url,
                aws_access_key_id=self._aws_access_key_id,
                aws_secret_access_key=self._aws_secret_access_key,
                aws_session_token=self._aws_session_token,
                region_name=self._region_name,
            )
        return self._resource

    def set_resource(
        self,
        connection_type=None,
        endpoint_url=None,
        aws_access_key_id=None,
        aws_secret_access_key=None,
        aws_session_token=None,
        region_name=None,
    ):
        """
        Sets a new resource.

        Args:
            connection_type: V1ConnectionType. The connection name to resolve.
            endpoint_url: `str`. The complete URL to use for the constructed client.
            aws_access_key_id: `str`. The access key to use when creating the client.
            aws_secret_access_key: `str`. The secret key to use when creating the client.
            aws_session_token: `str`. The session token to use when creating the client.
            region_name: `str`. The name of the region associated with the client.
                A client is associated with a single region.

        Returns:
             Service resource instance
        """
        connection_type = connection_type or self._connection_type
        connection_name = connection_type.name if connection_type else None
        context_path = get_connection_context_path(name=connection_name)
        self._resource = get_aws_resource(
            self.RESOURCE_TYPE or self.resource,
            endpoint_url=endpoint_url,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token,
            region_name=region_name,
            context_path=context_path,
        )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import requests

from polyaxon import settings
from polyaxon.exceptions import (
    HTTP_ERROR_MESSAGES_MAPPING,
    PolyaxonClientException,
    PolyaxonShouldExitError,
)
from polyaxon.logger import logger


class HttpTransportMixin:
    """HTTP operations transport."""

    @property
    def session(self):
        if not hasattr(self, "_session"):
            self._session = requests.Session()
        return self._session

    def request(
        self,
        method,
        url,
        params=None,
        data=None,
        files=None,
        json=None,  # noqa
        timeout=None,
        headers=None,
        session=None,
    ):
        """Send a request with the given data as json to the given URL.

        Args:
            method: HTTP method
            url: The url to send the request to.
            params: Dictionary of values to format url.
            data:
            files:
            json:
            timeout:
            headers: extra headers to add to the request.

        Returns:
            Request response if successful, Exception otherwise.

        Raises:
            PolyaxonHTTPError or one of its subclasses.
        """
        logger.debug(
            "Starting request to url: %s with params: %s, data: %s", url, params, data
        )

        request_headers = self._get_headers(headers=headers)
        timeout = timeout if timeout is not None else settings.LONG_REQUEST_TIMEOUT
        session = session or self.session

        try:
            response = session.request(
                method,
                url,
                params=params,
                data=data,
                json=json,
                headers=request_headers,
                files=files,
                timeout=timeout,
                verify=self.config.verify_ssl,
            )
        except (
            requests.exceptions.RequestException,
            requests.exceptions.Timeout,
            requests.exceptions.HTTPError,
        ) as exception:
            try:
                logger.debug("Exception: %s", exception, exc_info=True)
            except TypeError:
                pass
            raise PolyaxonShouldExitError(
                "Error connecting to Polyaxon server on `{}`.\n"
                "An Error `{}` occurred.\n"
                "Check your host and ports configuration "
                "and your internet connection.".format(url, exception)
            )

        logger.debug(
            "Response Content: %s, Headers: %s", response.content, response.headers
        )
        self.check_response_status(response, url)
        return response

    def check_response_status(self, response, endpoint):
        """Check if response is successful. Else raise Exception."""

        if 200 <= response.status_code < 300:
            return response

        try:
            logger.error(
                "Request to %s failed with status code %s. \n" "Reason: %s",
                endpoint,
                response.status_code,
                response.text,
            )
        except TypeError:
            logger.error("Request to %s failed with status code", endpoint)

        raise PolyaxonClientException(
            HTTP_ERROR_MESSAGES_MAPPING.get(response.status_code)
        )

    def get(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
        session=None,
    ):
        """Call request with a get."""
        return self.request(
            "GET",
            url=url,
            params=params,
            data=data,
            files=files,
            json=json_data,
            timeout=timeout,
            headers=headers,
            session=session,
        )

    def post(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
        session=None,
    ):
        """Call request with a post."""
        return self.request(
            "POST",
            url=url,
            params=params,
            data=data,
            files=files,
            json=json_data,
            timeout=timeout,
            headers=headers,
            session=session,
        )

    def patch(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
        session=None,
    ):
        """Call request with a patch."""
        return self.request(
            "PATCH",
            url=url,
            params=params,
            data=data,
            files=files,
            json=json_data,
            timeout=timeout,
            headers=headers,
            session=session,
        )

    def delete(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
        session=None,
    ):
        """Call request with a delete."""
        return self.request(
            "DELETE",
            url=url,
            params=params,
            data=data,
            files=files,
            json=json_data,
            timeout=timeout,
            headers=headers,
            session=session,
        )

    def put(
        self,
        url,
        params=None,
        data=None,
        files=None,
        json_data=None,
        timeout=None,
        headers=None,
        session=None,
    ):
        """Call request with a put."""
        return self.request(
            "PUT",
            url=url,
            params=params,
            data=data,
            files=files,
            json=json_data,
            timeout=timeout,
            headers=headers,
            session=session,
        )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.exceptions import PQLException
from polyaxon.pql.parser import (
    QueryOpSpec,
    parse_datetime_operation,
    parse_expression,
    parse_field,
    parse_negation_operation,
    parse_scalar_operation,
    parse_value_operation,
    split_query,
    tokenize_query,
)
from tests.utils import BaseTestCase


class TestParser(BaseTestCase):
    def test_base_parser_raises_for_invalid_expressions(self):
        with self.assertRaises(PQLException):
            parse_expression("foo:bar:moo")

        with self.assertRaises(PQLException):
            parse_expression("foo")

        with self.assertRaises(PQLException):
            parse_expression(None)

        with self.assertRaises(PQLException):
            parse_expression(12)

        with self.assertRaises(PQLException):
            parse_expression("fff:")

        with self.assertRaises(PQLException):
            parse_expression(":dsf")

        with self.assertRaises(PQLException):
            parse_expression(":")

    def test_base_parser_passes_for_valid_expressions(self):
        assert parse_expression("foo:bar") == ("foo", "bar")
        assert parse_expression("foo:>=bar") == ("foo", ">=bar")
        assert parse_expression("foo:bar|moo|boo") == ("foo", "bar|moo|boo")
        assert parse_expression("foo:bar..moo") == ("foo", "bar..moo")
        assert parse_expression("foo:~bar") == ("foo", "~bar")

        # Handles spaces
        assert parse_expression(" foo: bar ") == ("foo", "bar")
        assert parse_expression("foo :>=bar ") == ("foo", ">=bar")
        assert parse_expression(" foo :bar|moo|boo") == ("foo", "bar|moo|boo")
        assert parse_expression(" foo : bar..moo ") == ("foo", "bar..moo")
        assert parse_expression(" foo : ~bar ") == ("foo", "~bar")

    def test_parse_negation_operation(self):
        assert parse_negation_operation("foo") == (False, "foo")
        assert parse_negation_operation("~foo") == (True, "foo")
        assert parse_negation_operation("foo..boo") == (False, "foo..boo")
        assert parse_negation_operation("~foo..boo") == (True, "foo..boo")
        assert parse_negation_operation(">=foo") == (False, ">=foo")
        assert parse_negation_operation("~>=foo") == (True, ">=foo")
        assert parse_negation_operation("foo|boo") == (False, "foo|boo")
        assert parse_negation_operation("~foo|boo") == (True, "foo|boo")
        assert parse_negation_operation(" ~ >=foo ") == (True, ">=foo")
        assert parse_negation_operation(" foo|boo ") == (False, "foo|boo")
        assert parse_negation_operation("~ foo|boo") == (True, "foo|boo")

    def test_parse_datetime_operation(self):
        # Raises for not allowed operators
        with self.assertRaises(PQLException):
            parse_datetime_operation("foo|bar")

        with self.assertRaises(PQLException):
            parse_datetime_operation("")

        with self.assertRaises(PQLException):
            parse_datetime_operation("~")

        with self.assertRaises(PQLException):
            parse_datetime_operation("..")

        with self.assertRaises(PQLException):
            parse_datetime_operation("..da")

        with self.assertRaises(PQLException):
            parse_datetime_operation("asd..")

        with self.assertRaises(PQLException):
            parse_datetime_operation("asd..asd..asd")

        # Parses ranges
        assert parse_datetime_operation("foo..bar") == (
            QueryOpSpec("..", False, ["foo", "bar"])
        )
        assert parse_datetime_operation(" foo .. bar ") == (
            QueryOpSpec("..", False, ["foo", "bar"])
        )
        assert parse_datetime_operation("~ foo .. bar ") == (
            QueryOpSpec("..", True, ["foo", "bar"])
        )

        # Comparison
        assert parse_datetime_operation(">=foo") == (QueryOpSpec(">=", False, "foo"))
        assert parse_datetime_operation(" ~ <= bar ") == (
            QueryOpSpec("<=", True, "bar")
        )
        assert parse_datetime_operation("~ > bar ") == (QueryOpSpec(">", True, "bar"))

        # Equality
        assert parse_datetime_operation("foo") == (QueryOpSpec("=", False, "foo"))
        assert parse_datetime_operation(" ~  bar ") == (QueryOpSpec("=", True, "bar"))
        assert parse_datetime_operation("~bar") == (QueryOpSpec("=", True, "bar"))

    def test_parse_scalar_operation(self):
        # Raises for not allowed operators
        with self.assertRaises(PQLException):
            parse_scalar_operation("1|12")

        with self.assertRaises(PQLException):
            parse_scalar_operation("0.1..0.2")

        # Raise for not scalars
        with self.assertRaises(PQLException):
            parse_scalar_operation(">=f")

        with self.assertRaises(PQLException):
            parse_scalar_operation(" ~ <=f1 ")

        with self.assertRaises(PQLException):
            parse_scalar_operation("~ > bbb ")

        with self.assertRaises(PQLException):
            parse_datetime_operation("")

        with self.assertRaises(PQLException):
            parse_datetime_operation("~")

        with self.assertRaises(PQLException):
            parse_datetime_operation(">")

        # Comparison
        assert parse_scalar_operation(">=1") == (QueryOpSpec(">=", False, 1))
        assert parse_scalar_operation(" ~ <= 0.1 ") == (QueryOpSpec("<=", True, 0.1))
        assert parse_scalar_operation("~ > 20 ") == (QueryOpSpec(">", True, 20))

        # Equality
        assert parse_scalar_operation("1") == (QueryOpSpec("=", False, 1))
        assert parse_scalar_operation(" ~  2 ") == (QueryOpSpec("=", True, 2))
        assert parse_scalar_operation("~0.1") == (QueryOpSpec("=", True, 0.1))

    def test_parse_value_operation(self):
        # Raises for not allowed operators
        with self.assertRaises(PQLException):
            parse_value_operation("0.1..0.2")

        with self.assertRaises(PQLException):
            parse_datetime_operation("")

        with self.assertRaises(PQLException):
            parse_datetime_operation("~")

        # Raises for comparison
        with self.assertRaises(PQLException):
            parse_value_operation(">=f")

        with self.assertRaises(PQLException):
            parse_value_operation(" ~ <=f1 ")

        with self.assertRaises(PQLException):
            parse_value_operation("~|")

        with self.assertRaises(PQLException):
            parse_value_operation("|")

        with self.assertRaises(PQLException):
            parse_value_operation("~tag1 |")

        # Equality
        assert parse_value_operation("tag") == (QueryOpSpec("=", False, "tag"))
        assert parse_value_operation(" ~  tag ") == (QueryOpSpec("=", True, "tag"))
        assert parse_value_operation("~tag") == (QueryOpSpec("=", True, "tag"))

        # In op
        assert parse_value_operation("tag1|tag2") == (
            QueryOpSpec("|", False, ["tag1", "tag2"])
        )
        assert parse_value_operation(" ~  tag1|tag2 ") == (
            QueryOpSpec("|", True, ["tag1", "tag2"])
        )
        assert parse_value_operation("~tag1 | tag2| tag23") == (
            QueryOpSpec("|", True, ["tag1", "tag2", "tag23"])
        )

    def test_split_query(self):
        with self.assertRaises(PQLException):
            split_query("")

        with self.assertRaises(PQLException):
            split_query(",")

        with self.assertRaises(PQLException):
            split_query(", , ")

        assert len(split_query("name:~tag1 | tag2| tag23")) == 1
        assert len(split_query("name:~tag1 | tag2| tag23, name2:foo")) == 2

    def test_tokenize_query(self):
        with self.assertRaises(PQLException):
            tokenize_query("")

        with self.assertRaises(PQLException):
            tokenize_query(",")

        with self.assertRaises(PQLException):
            tokenize_query(", , ")

        assert tokenize_query("name:~tag1 | tag2| tag23") == {
            "name": ["~tag1 | tag2| tag23"]
        }
        assert tokenize_query(
            "name1:~tag1 | tag2| tag23, name1:foo, name2:sdf..dsf"
        ) == {"name1": ["~tag1 | tag2| tag23", "foo"], "name2": ["sdf..dsf"]}

    def test_parse_field(self):
        with self.assertRaises(PQLException):
            parse_field("")

        with self.assertRaises(PQLException):
            parse_field(".")

        with self.assertRaises(PQLException):
            parse_field("sdf.sdf.sf")

        with self.assertRaises(PQLException):
            parse_field("foo.")

        assert parse_field("foo") == ("foo", None)
        assert parse_field("foo_bar") == ("foo_bar", None)
        assert parse_field("foo.bar") == ("foo", "bar")
        assert parse_field("metric.foo_bar") == ("metric", "foo_bar")



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon import tracking
from polyaxon.exceptions import PolyaxonClientException
from polyaxon.tracking.contrib.tensorboard import PolyaxonTensorboardLogger

try:
    import tensorflow as tf
except ImportError:
    raise PolyaxonClientException(
        "tensorflow is required to use PolyaxonLoggingTensorHook"
    )

LoggingTensorHook = None
SessionRunHook = None

try:
    from tensorflow.train import LoggingTensorHook, SessionRunHook  # noqa
except ImportError:
    pass

try:
    from tensorflow.estimator import LoggingTensorHook, SessionRunHook  # noqa
except ImportError:
    pass

if not LoggingTensorHook or not SessionRunHook:
    raise PolyaxonClientException(
        "tensorflow is required to use PolyaxonLoggingTensorHook"
    )


class PolyaxonLoggingTensorHook(LoggingTensorHook):
    """Hook that logs data to console and Polyaxon"""

    def __init__(self, tensors, run=None, every_num_iterations=None, every_n_secs=None):
        super().__init__(
            tensors=tensors,
            every_num_iterations=every_num_iterations,
            every_n_secs=every_n_secs,
        )
        self.run = tracking.get_or_create_run(run)

    def _log_tensors(self, tensor_values):
        super()._log_tensors(tensor_values)

        if not self.run:
            return
        metrics = {k: tensor_values[k] for k in self._tensors.keys()}
        self.run.log_metrics(**metrics)


class PolyaxonSessionRunHook(SessionRunHook):
    def __init__(self, summary_op=None, steps_per_log=1000, run=None):
        self._summary_op = summary_op
        self._steps_per_log = steps_per_log
        self.run = tracking.get_or_create_run(run)

    def begin(self):
        if self._summary_op is None:
            self._summary_op = tf.summary.merge_all()
        self._step = -1

    def before_run(self, run_context):
        self._step += 1
        return tf.train.SessionRunArgs({"summary": self._summary_op})

    def after_run(self, run_context, run_values):
        if self._step % self._steps_per_log == 0:
            PolyaxonTensorboardLogger.process_summary(
                run_values.results["summary"], run=self.run
            )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys

import click

from polyaxon import settings
from polyaxon.cli.errors import handle_cli_error
from polyaxon.deploy.schemas.deployment_types import DeploymentTypes
from polyaxon.managers.cli import CliConfigManager
from polyaxon.managers.client import ClientConfigManager
from polyaxon.utils.formatting import Printer


@click.command()
@click.option(
    "-p", "--port", type=int, help="The port to expose the gateway, default to 8000"
)
@click.option(
    "-n",
    "--namespace",
    type=str,
    help="The namespace used for deploying Polyaxon, default polyaxon.",
)
@click.option(
    "-t",
    "--deployment-type",
    help="Deployment type.",
)
@click.option(
    "-r",
    "--release-name",
    type=str,
    help="The release name used for deploying Polyaxon, default polyaxon.",
)
def port_forward(port, namespace, deployment_type, release_name):
    """If you deploy Polyaxon using ClusterIP, you can use this command
    to access the gateway through `localhost:port`.
    """
    from polyaxon.deploy.operators.kubectl import KubectlOperator

    if not port and deployment_type in [
        DeploymentTypes.MICRO_K8S,
        DeploymentTypes.MINIKUBE,
    ]:
        port = 31833
    port = port or 8000
    namespace = namespace or "polyaxon"
    release_name = release_name or "polyaxon"

    kubectl = KubectlOperator()
    args = [
        "port-forward",
        "-n",
        namespace,
        "svc/{}-polyaxon-gateway".format(release_name),
        "{}:80".format(port),
    ]

    try:
        _config = ClientConfigManager.get_config_or_default()
    except Exception as e:
        handle_cli_error(e, message="Polyaxon load configuration.")
        Printer.print_header(
            "You can reset your config by running: polyaxon config purge"
        )
        sys.exit(1)

    _config.host = "http://localhost:{}".format(port)
    ClientConfigManager.set_config(_config)
    CliConfigManager.purge()
    Printer.print_header("Client configuration is updated!")
    Printer.print_success("Polyaxon will be available at: {}".format(_config.host))
    stdout = kubectl.execute(
        args=args, is_json=False, stream=settings.CLIENT_CONFIG.debug
    )
    click.echo(stdout)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from mock import patch

from polyaxon import settings
from polyaxon.client import RunClient
from polyaxon.stores.polyaxon_store import PolyaxonStore
from tests.utils import BaseTestCase


class TestPolyaxonStore(BaseTestCase):
    def test_download_file(self):
        store = PolyaxonStore(
            client=RunClient(owner="test", project="test", run_uuid="uid")
        )
        with patch(
            "polyaxon.stores.polyaxon_store.PolyaxonStore.download"
        ) as mock_call:
            result = store.download_file(url="url", path="test/path")

        assert result == "{}/uid/test/path".format(settings.CLIENT_CONFIG.archive_root)
        assert mock_call.call_count == 1
        assert mock_call.call_args_list[0][1] == {
            "filename": result,
            "params": {"path": "test/path"},
            "url": "url",
        }

        with patch(
            "polyaxon.stores.polyaxon_store.PolyaxonStore.download"
        ) as mock_call:
            result = store.download_file(url="url", path="test/path", untar=False)

        assert result == "{}/uid/test/path.tar.gz".format(
            settings.CLIENT_CONFIG.archive_root
        )
        assert mock_call.call_count == 1
        assert mock_call.call_args_list[0][1] == {
            "filename": result,
            "untar": False,
            "params": {"path": "test/path"},
            "url": "url",
        }

        with patch(
            "polyaxon.stores.polyaxon_store.PolyaxonStore.download"
        ) as mock_call:
            result = store.download_file(url="url", path="test/path", untar=True)

        assert result == "{}/uid/test/path".format(settings.CLIENT_CONFIG.archive_root)
        assert mock_call.call_count == 1
        assert mock_call.call_args_list[0][1] == {
            "filename": "{}.tar.gz".format(result),
            "untar": True,
            "params": {"path": "test/path"},
            "url": "url",
        }



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from collections import OrderedDict
from typing import Dict, List

from polyaxon.cli.errors import handle_cli_error
from polyaxon.config_reader.spec import ConfigSpec
from polyaxon.exceptions import PolyaxonfileError
from polyaxon.polyaxonfile.manager import (
    get_op_specification,
    is_supported_in_eager_mode,
)
from polyaxon.polyaxonfile.params import parse_params
from polyaxon.polyaxonfile.specs import get_specification, kinds
from polyaxon.polyflow import V1Init, V1Operation
from polyaxon.utils.formatting import Printer, dict_tabulate
from polyaxon.utils.list_utils import to_list


def collect_references(config: V1Operation, path_context: str = None):
    if config.has_component_reference or (
        config.has_hub_reference and not config.has_public_hub_reference
    ):
        return config
    elif config.has_public_hub_reference:
        component = ConfigSpec.get_from(config.hub_ref, "hub").read()
    elif config.has_url_reference:
        component = ConfigSpec.get_from(config.url_ref, "url").read()
    elif config.has_path_reference:
        path_ref = config.path_ref
        if path_context:
            path_ref = os.path.join(
                os.path.dirname(os.path.abspath(path_context)), path_ref
            )
        component = ConfigSpec.get_from(path_ref).read()
    else:
        raise PolyaxonfileError("Operation found without component")

    component = get_specification(data=component)
    if component.kind != kinds.COMPONENT:
        if config.has_url_reference:
            ref_type = "Url ref"
            ref = config.url_ref
        else:
            ref_type = "Path ref"
            ref = config.path_ref
        raise PolyaxonfileError(
            "the reference ({}) `{}` is of kind `{}`, it should be a `{}`".format(
                ref, ref_type, component.kind, kinds.COMPONENT
            )
        )
    config.component = component
    return config


def check_polyaxonfile(
    polyaxonfile: str = None,
    python_module: str = None,
    url: str = None,
    hub: str = None,
    params: Dict = None,
    presets: List[str] = None,
    queue: str = None,
    nocache: bool = None,
    cache: bool = None,
    verbose: bool = True,
    eager_hub: bool = True,
    is_cli: bool = True,
    to_op: bool = True,
    validate_params: bool = True,
    eager: bool = False,
    git_init: V1Init = None,
    ignore_template: bool = False,
):
    if sum([1 for i in [python_module, url, hub] if i]) > 1:
        message = (
            "You can only use one and only one option: "
            "hub, url, or a python module.".format(hub)
        )
        if is_cli:
            Printer.print_error(message, sys_exit=True)
        else:
            raise PolyaxonfileError(message)
    if not any([polyaxonfile, python_module, url, hub]):
        polyaxonfile = check_default_path(path=".")
    if not any([polyaxonfile, python_module, url, hub]):
        message = (
            "Something went wrong, `check_polyaxonfile` was called without a polyaxonfile, "
            "a hub component reference, a url or a python module."
        )
        if is_cli:
            Printer.print_error(message, sys_exit=True)
        else:
            raise PolyaxonfileError(message)
    if hub and not to_op:
        message = "Something went wrong, calling hub component `{}` without operation.".format(
            hub
        )
        if is_cli:
            Printer.print_error(message, sys_exit=True)
        else:
            raise PolyaxonfileError(message)

    polyaxonfile = to_list(polyaxonfile, check_none=True)

    parsed_params = None
    if params:
        parsed_params = parse_params(params, is_cli=is_cli)

    if not any([os.path.isfile(f) for f in polyaxonfile]) and not any(
        [python_module, url, hub]
    ):
        message = "Please pass a valid polyaxonfile, a python module, a url, or a component name"
        if is_cli:
            Printer.print_error(message, sys_exit=True)
        else:
            raise PolyaxonfileError(message)

    try:
        plx_file = None
        path_context = None
        public_hub = hub and "/" not in hub

        if not hub or (public_hub and eager_hub):
            if python_module:
                path_context = python_module
                plx_file = (
                    ConfigSpec.get_from(python_module, config_type=".py")
                    .read()
                    .to_dict(include_kind=True, include_version=True)
                )

            elif url:
                plx_file = ConfigSpec.get_from(url, "url").read()

            elif hub:
                plx_file = ConfigSpec.get_from(hub, "hub").read()

            else:
                path_context = polyaxonfile.pop(0)
                plx_file = ConfigSpec.read_from(path_context)

            plx_file = get_specification(data=plx_file)
            if plx_file.kind == kinds.OPERATION:
                plx_file = collect_references(plx_file, path_context)

        if to_op or hub:
            plx_file = get_op_specification(
                hub=hub,
                config=plx_file,
                params=parsed_params,
                presets=presets,
                queue=queue,
                nocache=nocache,
                cache=cache,
                path_context=path_context,
                validate_params=validate_params,
                preset_files=polyaxonfile,
                git_init=git_init,
            )
        if verbose and is_cli:
            Printer.print_success("Polyaxonfile valid")
        if ignore_template:
            plx_file.disable_template()
        if plx_file.is_template():
            template_message = "This polyaxonfile was marked as template by the owner:"
            if plx_file.template.description:
                template_message += "\ntemplate description: {}".format(
                    plx_file.template.description
                )
            if plx_file.template.fields:
                template_message += "\ntemplate fields that need changes: {}".format(
                    plx_file.template.fields
                )
            Printer.print_warning(template_message)
        if eager:
            is_supported_in_eager_mode(spec=plx_file)
        return plx_file
    except Exception as e:
        message = "Polyaxonfile is not valid."
        if is_cli:
            handle_cli_error(e, message=message, sys_exit=True)
        else:
            raise PolyaxonfileError(message) from e


def check_polyaxonfile_kind(specification, kind):
    if specification.kind != kind:
        Printer.print_error(
            "Your polyaxonfile must be of kind: `{}`, "
            "received: `{}`.".format(kind, specification.kind),
            sys_exit=True,
        )


def get_matrix_info(kind, concurrency, early_stopping=False, **kwargs):
    info = OrderedDict()
    info["Matrix kind"] = kind.lower()
    info["Concurrency"] = (
        "{} runs".format("sequential")
        if concurrency == 1
        else "{} concurrent runs".format(concurrency)
    )
    info["Early stopping"] = "activated" if early_stopping else "deactivated"
    if "num_runs" in kwargs:
        info["Num of runs to create"] = kwargs["num_runs"]

    dict_tabulate(info)


DEFAULT_POLYAXON_FILE_NAME = [
    "polyaxon",
    "polyaxonci",
    "polyaxon-ci",
    "polyaxon.ci",
    "polyaxonfile",
]

DEFAULT_POLYAXON_FILE_EXTENSION = ["yaml", "yml", "json"]


def check_default_path(path):
    path = os.path.abspath(path)
    for filename in DEFAULT_POLYAXON_FILE_NAME:
        for ext in DEFAULT_POLYAXON_FILE_EXTENSION:
            filepath = os.path.join(path, "{}.{}".format(filename, ext))
            if os.path.isfile(filepath):
                return filepath



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import polyaxon_sdk

from marshmallow import fields

from polyaxon.polyflow.params import ParamSchema
from polyaxon.schemas.base import BaseCamelSchema, BaseConfig


class ActionSchema(BaseCamelSchema):
    hub_ref = fields.Str(required=True)
    label = fields.Str(allow_none=True)
    many = fields.Bool(allow_none=True)
    params = fields.Dict(
        keys=fields.Str(), values=fields.Nested(ParamSchema), allow_none=True
    )
    run_patch = fields.Dict(keys=fields.Str(), values=fields.Raw(), allow_none=True)

    @staticmethod
    def schema_config():
        return V1Action


class V1Action(BaseConfig, polyaxon_sdk.V1Action):
    """In order to extend Polyaxon UI and CLI, you can set actions on your operations.
    Every action is a reference to a component that can be executed
    based on the context of the operation where it's defined

    **Use cases**

     * A training job can have Tensorboard as an action, this will automatically
    add a UI button in the dashboard so that you can start a Tensorboard based
    on the outputs of the run.
     * Streamlit app that can consume the outputs of some specific jobs.
     * Papermill for parametrized notebooks.


    **UI**

    Polyaxon UI will display up-to 3 actions in the run's dropdown,
    and allow users to view all of them (if there are more than 3),
    users interacting with the dashboard can start an action by clicking the buttons.

    **CLI**

    Polyaxon CLI will also allow to start actions in a much usable than passing
    params to another component, and it will make the CLI much usable,
    for instance, this is the behavior for starting a Keras operation and running a Tensorboard:

    ```bash
    polyaxon run -f polyaxonfile.yaml
    polyaxon run --hub tensorboard --uuid UUID
    ```

    By adding actions to a specific training component,
    the CLI can leverage the context of a run to resolve the parameters automatically:

    ```bash
    polyaxon run -f polyaxonfile.yaml
    polyaxon ops start --action=tensorboard
    ```

    **Comparison**

    Actions will be indexed as special tags to allow users to filter all
    runs based on an action or multiple.
    For example filtering all runs with a Tensorboard action,
    the table comparison will also provide an action for the group.

     Args:
         hub_ref: str
         label: str, optional
         many: str, bool
         params: Dict[str, [V1Param](/docs/core/specification/params/)], optional
         run_patch: Dict, optional

    ## YAML usage

    ```yaml
    >>> notification:
    >>>   hubRef: tensorboard
    >>>   label: "Start Tensorboard"
    ```

    ## Python usage

    ```python
    >>> from polyaxon.polyflow import V1Action
    >>> action = V1Action(
    >>>     hub_ref="tensorboard",
    >>>     label="Start Tensorboard",
    >>> )
    ```

    ## Fields

    ### hubRef

    Polyaxon provides a [Component Hub](/docs/management/component-hub/)
    for hosting versioned components with an access control system to improve
    the productivity of your team.

    To trigger a hook based on a component hosted on Polyaxon Component Hub, you can use `hubRef`

    ```yaml
    >>> action:
    >>>   hubRef: slack:v1
    ...
    ```

    Or custom hook component

    ```yaml
    >>> action:
    >>>   hubRef:  my-component:dev
    ...
    ```

    ### label

    The UI label to use for the button, if not provided it will default to the component's name.

    ```yaml
    >>> action:
    >>>   label: "Start My App"
    ...
    ```

    ### many

    This is an optional boolean to tell Polyaxon if this action
    is expected to act on a single run or multiple runs.

    ```yaml
    >>> action:
    >>>   hubRef: tensorboard:multi-runs
    >>>   many: true
    ```

    ### params

    The [params](/docs/core/specification/params/) to pass if the handler requires extra params,
    they will be validated against the inputs/outputs.
    If a parameter is passed and the component does not define a corresponding inputs/outputs,
    a validation error will be raised unless the param has the contextOnly flag enabled.

    ```yaml
    >>> action:
    >>>   params:
    >>>     param1: {value: 1.1}
    >>>     param2: {value: test}
    >>>   ...
    ```

    ### runPatch

    Component's environment section can be [patched](/docs/core/specification/operation/#runPatch).

    Example patching the container resources:

    ```yaml
    >>> action:
    >>>   runPatch:
    >>>     container:
    >>>       resources:
    >>>         limits:
    >>>           cpu: 50m
    ```
    """

    IDENTIFIER = "action"
    SCHEMA = ActionSchema
    REDUCED_ATTRIBUTES = [
        "hubRef",
        "label",
        "many",
        "params",
        "runPatch",
    ]



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime

from polyaxon.exceptions import PolyaxonDateTimeFormatterException
from polyaxon.utils.date_utils import DateTimeFormatter
from tests.utils import BaseTestCase


class TestDateTimeFormatter(BaseTestCase):
    def test_format_date_works_as_expected_for_given_datetime(self):
        result = DateTimeFormatter.format_date(datetime(2012, 12, 24))
        self.assertEqual(result, "2012-12-24")

    def test_format_date_accepts_rasies_for_none(self):
        with self.assertRaises(PolyaxonDateTimeFormatterException):
            DateTimeFormatter.format_date(None)

    def test_format_datetime_works_as_expected_for_given_datetime(self):
        result = DateTimeFormatter.format_datetime(datetime(2012, 12, 24, 16, 17, 18))
        self.assertEqual(result, "2012-12-24 16:17:18")

    def test_format_datetime_accepts_raises_for_none(self):
        with self.assertRaises(PolyaxonDateTimeFormatterException):
            DateTimeFormatter.format_datetime(None)

    def test_extract_date_works_as_expected_for_given_date_string(self):
        result = DateTimeFormatter.extract_date("2012-12-24", timezone="Europe/Berlin")
        self.assertEqual(result.year, 2012)
        self.assertEqual(result.month, 12)
        self.assertEqual(result.day, 24)

    def test_extract_date_accepts_raises_for_none(self):
        with self.assertRaises(PolyaxonDateTimeFormatterException):
            DateTimeFormatter.extract_date(None, timezone="Europe/Berlin")

    def test_extract_datetime_rejects_invalid_date(self):
        with self.assertRaises(PolyaxonDateTimeFormatterException):
            DateTimeFormatter.extract_date("foo", timezone="Europe/Berlin")

    def test_extract_datetime_works_as_expected_for_given_datetime_string(self):
        result = DateTimeFormatter.extract_datetime(
            "2012-12-24 16:17:18", timezone="Europe/Berlin"
        )
        self.assertEqual(result.year, 2012)
        self.assertEqual(result.month, 12)
        self.assertEqual(result.day, 24)
        self.assertEqual(result.hour, 16)
        self.assertEqual(result.minute, 17)
        self.assertEqual(result.second, 18)

    def test_extract_datetime_accepts_raises_for_none(self):
        with self.assertRaises(PolyaxonDateTimeFormatterException):
            DateTimeFormatter.extract_datetime(None, timezone="Europe/Berlin")

    def test_extract_datetime_rejects_invalid_datetime(self):
        with self.assertRaises(PolyaxonDateTimeFormatterException):
            DateTimeFormatter.extract_datetime("foo", timezone="Europe/Berlin")

    def test_extract_works_as_expected_for_given_date_string(self):
        result = DateTimeFormatter.extract("2012-12-24", timezone="Europe/Berlin")
        self.assertEqual(result.year, 2012)
        self.assertEqual(result.month, 12)
        self.assertEqual(result.day, 24)

    def test_extract_works_as_expected_for_given_datetime_string(self):
        result = DateTimeFormatter.extract(
            "2012-12-24 16:17:18", timezone="Europe/Berlin"
        )
        self.assertEqual(result.year, 2012)
        self.assertEqual(result.month, 12)
        self.assertEqual(result.day, 24)
        self.assertEqual(result.hour, 16)
        self.assertEqual(result.minute, 17)
        self.assertEqual(result.second, 18)

    def test_extract_accepts_raises_for_none(self):
        with self.assertRaises(PolyaxonDateTimeFormatterException):
            DateTimeFormatter.extract(None, timezone="Europe/Berlin")

    def test_extract_rejects_invalid_datetime(self):
        with self.assertRaises(PolyaxonDateTimeFormatterException):
            DateTimeFormatter.extract("foo", timezone="Europe/Berlin")



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from polyaxon.constants import DEFAULT
from polyaxon.exceptions import PolyaxonClientException
from polyaxon.managers.project import ProjectConfigManager
from polyaxon.utils.formatting import Printer


def get_project_full_name(owner: str = None, project: str = None) -> str:
    if owner and project:
        return "{}/{}".format(owner, project)
    return project


def get_project_info(project):
    from polyaxon import settings

    parts = project.replace(".", "/").split("/")
    if len(parts) == 2:
        owner, project_name = parts
    else:
        owner = settings.AUTH_CONFIG.username
        project_name = project

    return owner, project_name


def get_project_or_local(project=None, is_cli: bool = False):
    from polyaxon import settings

    if not project and not ProjectConfigManager.is_initialized():
        if is_cli:
            Printer.print_error("Please provide a valid project.")
            sys.exit(1)
        else:
            raise PolyaxonClientException("Please provide a valid project.")

    if project:
        owner, project_name = get_project_info(project)
    else:
        try:
            project = ProjectConfigManager.get_config()
        except TypeError:
            Printer.print_error(
                "Found an invalid project config or project config cache, "
                "if you are using Polyaxon CLI please run: "
                "`polyaxon config purge --cache-only`",
                sys_exit=True,
            )

        owner, project_name = project.owner, project.name

    if not owner and (not settings.CLI_CONFIG or settings.CLI_CONFIG.is_ce):
        owner = DEFAULT

    if not all([owner, project_name]):
        if is_cli:
            Printer.print_error("Please provide a valid project.")
            sys.exit(1)
        else:
            raise PolyaxonClientException("Please provide a valid project.")
    return owner, project_name



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import fields, validate

from polyaxon.polyflow.component.base import BaseComponent, BaseComponentSchema
from polyaxon.polyflow.matrix import MatrixMixin, MatrixSchema
from polyaxon.polyflow.schedules import ScheduleMixin, ScheduleSchema
from polyaxon.polyflow.trigger_policies import V1TriggerPolicy


class BaseOpSchema(BaseComponentSchema):
    schedule = fields.Nested(ScheduleSchema, allow_none=True)
    events = fields.List(fields.Str(), allow_none=True)
    matrix = fields.Nested(MatrixSchema, allow_none=True)
    dependencies = fields.List(fields.Str(), allow_none=True)
    trigger = fields.Str(
        allow_none=True, validate=validate.OneOf(V1TriggerPolicy.allowable_values)
    )
    conditions = fields.Str(allow_none=True)
    skip_on_upstream_skip = fields.Bool(allow_none=True)

    @staticmethod
    def schema_config():
        return BaseOp


class BaseOp(BaseComponent, MatrixMixin, ScheduleMixin):
    SCHEMA = BaseOpSchema
    REDUCED_ATTRIBUTES = BaseComponent.REDUCED_ATTRIBUTES + [
        "schedule",
        "events",
        "matrix",
        "dependencies",
        "trigger",
        "conditions",
        "skipOnUpstreamSkip",
    ]

    FIELDS_SAME_KIND_PATCH = ["schedule", "matrix"]

    def get_matrix_kind(self):
        return self.matrix.kind if self.matrix else None

    def get_schedule_kind(self):
        return self.schedule.kind if self.schedule else None



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from datetime import datetime
from typing import Optional

from polyaxon.containers.contexts import CONTEXT_MOUNT_ARTIFACTS_FORMAT
from polyaxon.stores.manager import get_artifacts_connection, upload_file_or_dir
from polyaxon.utils.date_utils import path_last_modified


def sync_artifacts(last_check: Optional[datetime], run_uuid: str):
    connection_type = get_artifacts_connection()
    path_from = CONTEXT_MOUNT_ARTIFACTS_FORMAT.format(run_uuid)
    new_check = path_last_modified(path_from)
    # check if there's a path to sync
    if os.path.exists(path_from):
        path_to = os.path.join(connection_type.store_path, run_uuid)

        upload_file_or_dir(
            path_from=path_from,
            path_to=path_to,
            is_file=False,
            workers=5,
            last_time=last_check,
            connection_type=connection_type,
            exclude=["plxlogs"],
        )

    return new_check



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon import settings


def _get_indent(indent):
    return (
        settings.PROXIES_CONFIG.nginx_indent_char
        * settings.PROXIES_CONFIG.nginx_indent_width
        * indent
    )


def get_config(options, indent=0, **kwargs):
    _options = options.format(**kwargs)
    config = []
    for p in _options.split("\n"):
        config.append("{}{}".format(_get_indent(indent), p))

    return clean_config(config)


def clean_config(config):
    return "\n".join(config).replace("    \n", "")
